// Copyright(C)SEGA 
#ifndef	COLLADA_STRUCT_H
#define COLLADA_STRUCT_H

#include "Collada/Enum.h"
#include <vector>
#include <string>

namespace Collada {

    // Floatx2\
    struct	Float2 {

        // RXgN^
        Float2() : x(0.0f), y(0.0f) {};

        // RXgN^
        Float2( const float xx, const float yy ) :
        x(xx), y(yy) {}

        // lݒ
        void set( const float xx, const float yy ) { x = xx; y = yy; }

        // l        
        float	x, y;

    };

    // Floatx3\
    struct	Float3 {

        // RXgN^
        Float3() : x(0.0f), y(0.0f), z(0.0f) {};

        // RXgN^
        Float3( const float xx, const float yy, const float zz ) :
        x(xx), y(yy), z(zz) {}

        // lݒ
        void set( const float xx, const float yy, const float zz ) { x = xx; y = yy; z = zz; }        

        // l        
        float	x, y, z;

    };

    // Floatx4\
    struct	Float4 {

        // RXgN^
        Float4() : x(0.0f), y(0.0f), z(0.0f), w(0.0f) {};

        // RXgN^
        Float4( const float xx, const float yy, const float zz, const float ww ) :
        x(xx), y(yy), z(zz), w(ww) {}

        // lݒ
        void set( const float xx, const float yy, const float zz, const float ww ) { x = xx; y = yy; z = zz; w = ww; }

        // l        
        float	x, y, z, w;

    };

    // Matrix(4x4)\
    struct	Matrix {

        // RXgN^
        Matrix() {
            m[ 1] = m[ 2] = m[ 3] = 0.0f;
            m[ 4] = m[ 6] = m[ 7] = 0.0f;
            m[ 8] = m[ 9] = m[11] = 0.0f;
            m[12] = m[13] = m[14] = 0.0f;            
            m[ 0] = m[ 5] = m[10] = m[15] = 1.0f;
        }

        // RXgN^        
        Matrix( float input[] ) {

            for( int i=0; i<16; i++ ) {

                m[i] = input[i];

            }

        }

        // RXgN^
        Matrix( float i00, float i01, float i02, float i03,
            float i10, float i11, float i12, float i13,
            float i20, float i21, float i22, float i23,
            float i30, float i31, float i32, float i33 ) {

                m[ 0] = i00; m[ 1] = i01; m[ 2] = i02; m[ 3] = i03;
                m[ 4] = i10; m[ 5] = i11; m[ 6] = i12; m[ 7] = i13;
                m[ 8] = i20; m[ 9] = i21; m[10] = i22; m[11] = i23;
                m[12] = i30; m[13] = i31; m[14] = i32; m[15] = i33;            

        }

        // l
        float	m[16];

    };

    // J[\(float)
    struct	Color {

        // RXgN^
        Color() : r(0.0f), g(0.0f), b(0.0f), a(0.0f) {};

        // RXgN^
        Color( const float rr, const float gg, const float bb, const float aa ) :
        r(rr), g(gg), b(bb), a(aa) {};

        // operator==
        bool operator==( const Color& c ) const {

            if( r == c.r &&
                g == c.g &&
                b == c.b &&
                a == c.a ) {
                    return true;
            }
            return false;
        }

        // operator!=
        bool operator!=( const Color& c ) const {

            if( *this == c ) {

                return false;

            }
            else {

                return true;

            }			 

        }

        // l
        float	r, g, b, a;

    };

    // Surface\
    struct	Surface {

    public:
        // RXgN^
        Surface()
        {
        }

        // RXgN^
        Surface( const std::string& s,
            const std::string& t,
            const std::string& i,
            const std::string& f ) : mSId( s ), mType( t ), mInitFrom( i ), mFormat( f )
        {
        }

        std::string mSId; // sid
        std::string mType; // SurfaceType
        std::string mInitFrom; // QƐImage
        std::string mFormat; // format

    };

    // Sampler2D\
    class Sampler2D {

    public:
        // RXgN^
        Sampler2D()
        {
        }

        // RXgN^
        Sampler2D( const std::string& i,
            const std::string& s ) : mSId( i ), mSource( s )
        {
        }

        std::string mSId; // sid    
        std::string mSource; // QƐsurface

    };

    // Unit\
    class Unit {

    public:
        // RXgN^
        Unit() : mMeter(0.01f), mName("centimeter")
        {
        };

        // RXgN^
        Unit( const float m, const char* n ) : mMeter( m ), mName( n )
        {
        };

        float mMeter; // P[g̒l
        std::string mName; // P

    };

    // Param\
    class Param {

    public:
        // RXgN^
        Param() 
        {
        }

        // RXgN^
        Param( const std::string& n, const std::string& t ) : mName( n ), mType( t )
        {
        }

        // O
        std::string mName;

        // ^
        std::string mType;

    };

    // channel\
    class Channel {

    public:
        // RXgN^	
        Channel()
        {
        }

        // RXgN^
        Channel( const std::string& s, const std::string& t ) : mSource( s ), mTarget( t )
        {
        }

        // source
        std::string mSource;

        // target
        std::string mTarget;

    };

    // input\
    class Input {

    public:
        // RXgN^
        Input() : mOffset( 0 ), mSet( 0 )
        {
        }

        // RXgN^        
        Input( const Semantic se,
            const std::string& so,
            const int off,
            const int set = 0 ) : mSemantic( se ), mSource( so ), mOffset( off ), mSet( set ) 
        {
        }

        // Z}eBbN
        Semantic mSemantic;

        // \[X
        std::string mSource;

        // ItZbg
        int mOffset;

        // Zbg
        int mSet;

    };

    // Agr[g
    class Attribute {

    public:
        // RXgN^
        Attribute() {}

        // RXgN^
        Attribute( const std::string& n, const std::string& v ) : mName( n ), mValue( v )
        {
        }

        // operator==
        bool operator==( const Attribute& a ) const 
        {
            if( mName != a.mName ) {

                return false;

            }
            if( mValue != a.mValue ) {

                return false;

            }
            return true;

        }

        // operator!=
        bool operator!=( const Attribute& a ) const
        {

            return ( *this == a ) ? false : true;

        }

        // O
        std::string mName;

        // l
        std::string mValue;

    };

    // CX^XVisualScene
    struct	InstanceVisualScene {

        // RXgN^
        InstanceVisualScene() {}

        // RXgN^
        InstanceVisualScene( const std::string& url ) : mURL( url ) {}

        // URL
        std::string mURL;

    };

    // CX^XCg
    struct	InstanceLight {

        // RXgN^
        InstanceLight();

        // RXgN^
        InstanceLight( const std::string& url ) : mURL( url ) {}

        // URL
        std::string mURL;

    };

    // CX^XJ
    struct	InstanceCamera {

        // RXgN^		
        InstanceCamera() {}

        // RXgN^	
        InstanceCamera( const std::string& url ) : mURL( url ) {}

        // URL
        std::string mURL;

    };

    // CX^XAj[V
    class InstanceAnimation {

    public:
        // RXgN^	
        InstanceAnimation() {}

        // RXgN^		
        InstanceAnimation( const std::string& url ) : mURL( url ) {}

        // URL
        std::string mURL;

    };

    // V[
    struct Scene {

        // CX^XVisualScene
        InstanceVisualScene instanceVisualScene;

    };

    // CX^X}eA
    class InstanceMaterial {

    public:
        // RXgN^
        InstanceMaterial( const std::string& s, const std::string& t ) : mSymbol( s ), mTarget( t )
        {
        }

        // RXgN^
        InstanceMaterial()
        {
        }

        // V{
        std::string mSymbol;

        // ^[Qbg
        std::string mTarget;

    };
}

#endif	// COLLADA_STRUCT_H
