// Copyright(C)SEGA 
#include "Collada/Animation.h"
#include "Collada/Macro.h"
#include "Collada/StructVector.h"
#include "Collada/Sampler.h"
#include "Collada/Util.h"
#include "Collada/XML.h"

namespace Collada {

    // RXgN^
    Animation::Animation() : 
mInput( NULL ),
mOutput( NULL ), 
mInterpolation( NULL ),
mInTangent( NULL ),
mOutTangent( NULL ),
mSampler( NULL )
{
}

// fXgN^
Animation::~Animation()
{
    SAFE_DELETE( mInput );
    SAFE_DELETE( mOutput );
    SAFE_DELETE( mInterpolation );
    SAFE_DELETE( mInTangent );
    SAFE_DELETE( mOutTangent );
    SAFE_DELETE( mSampler );
}

// XMLݒ
void Animation::toXML( XMLElement* e )
{
    e->mName = "animation";
    e->mAttributes.push_back( new XMLAttribute( "id", mId ) );
    {
        const int animationNumber = mAnimations.size();
        for( int i=0; i<animationNumber; i++ ) {

            XMLElement* animationE = new XMLElement();				
            mAnimations[ i ]->toXML( animationE );
            e->mChildren.push_back( animationE );

        }

        if( mInput ) {

            XMLElement* inputE = new XMLElement();
            mInput->toXML( inputE );
            e->mChildren.push_back( inputE );

        }

        if( mOutput ) {

            XMLElement* outputElement = new XMLElement();
            mOutput->toXML( outputElement );
            e->mChildren.push_back( outputElement );

        }

        if( mInterpolation ) {

            XMLElement* interpolationE = new XMLElement();
            mInterpolation->toXML( interpolationE );
            e->mChildren.push_back( interpolationE );

        }

        if( mInTangent ) {

            XMLElement* inTangentE = new XMLElement();
            mInTangent->toXML( inTangentE );
            e->mChildren.push_back( inTangentE );

        }

        if( mOutTangent ) {

            XMLElement* outTangentE = new XMLElement();
            mOutTangent->toXML( outTangentE );
            e->mChildren.push_back( outTangentE );

        }

        if( mSampler ) {

            XMLElement* samplerE = new XMLElement();
            mSampler->toXML( samplerE );
            e->mChildren.push_back( samplerE );

        }

        XMLElement* channelElement = new XMLElement();
        channelElement->mName = "channel";            
        channelElement->mAttributes.push_back( new XMLAttribute( "source", mChannel.mSource ) );
        channelElement->mAttributes.push_back( new XMLAttribute( "target", mChannel.mTarget ) );
        e->mChildren.push_back( channelElement );
    }
}
}