// Copyright(C)SEGA 
#include "Collada/Camera.h"
#include "Collada/Macro.h"
#include "Collada/Extra.h"
#include "Collada/Util.h"
#include "Collada/XML.h"

namespace Collada {

    // fXgN^
    Camera::~Camera()
    {
        SAFE_DELETE( mExtra );
    }

    // XMLݒ
    void Camera::toXML( XMLElement* e )
    {
        e->mName = "camera";
        e->mAttributes.push_back( new XMLAttribute( "id", mId ) );
        e->mAttributes.push_back( new XMLAttribute( "name", mName ) );
        {
            XMLElement* opticsE = new XMLElement();            
            opticsE->mName = "optics";            
            {
                XMLElement* techniqueCommonE = new XMLElement();
                techniqueCommonE->mName = "technique_common";
                opticsE->mChildren.push_back( techniqueCommonE );                
                {
                    XMLElement* perspectiveE = new XMLElement();
                    perspectiveE->mName = "perspective";                    
                    techniqueCommonE->mChildren.push_back( perspectiveE );
                    {
                        XMLElement* yfovE = new XMLElement();
                        XMLElement* aspectRatioE = new XMLElement();
                        XMLElement* znearE = new XMLElement();
                        XMLElement* zfarE = new XMLElement();

                        setFloatXML( "yfov", mYfov, yfovE );
                        setFloatXML( "aspect_ratio", mAspectRatio, aspectRatioE );
                        setFloatXML( "znear", mZNear, znearE );
                        setFloatXML( "zfar", mZFar, zfarE );

                        perspectiveE->mChildren.push_back( yfovE );
                        perspectiveE->mChildren.push_back( aspectRatioE );
                        perspectiveE->mChildren.push_back( znearE );
                        perspectiveE->mChildren.push_back( zfarE );
                    }
                }
            }

            XMLElement* extraE = new XMLElement();
            extraE->mName = "extra"; 
            e->mChildren.push_back( extraE );            
            {     
                XMLElement* techniqueE = new XMLElement();
                techniqueE->mName = "technique";                
                techniqueE->mAttributes.push_back( new XMLAttribute( "profile", "Softimage" ) );
                extraE->mChildren.push_back( techniqueE );
                {                    
                    // ̕ӂ̎͐Ȃ
                    // TODO: ǂ擾ďo͂邩ׂ邱
                    XMLElement* filmFitE = new XMLElement();
                    XMLElement* filmFitOffsetE = new XMLElement();
                    XMLElement* filmOffsetXE = new XMLElement();
                    XMLElement* filmOffsetYE = new XMLElement();
                    XMLElement* horizontalApertureE = new XMLElement();
                    XMLElement* lensSqueezeE = new XMLElement();                                                                                                 
                    XMLElement* verticalApertureE = new XMLElement();

                    // KȒlo͂Ă
                    // TODO: ۂɂ̓J擾lo
                    setFloatXML( "film_fit", 0.0f, filmFitE );
                    setFloatXML( "film_fit_offset", 0.0f, filmFitOffsetE );
                    setFloatXML( "film_offsetX", 0.0f, filmOffsetXE );
                    setFloatXML( "film_offsetY", 0.0f, filmOffsetYE );
                    setFloatXML( "horizontal_aperture", 0.0f, horizontalApertureE );
                    setFloatXML( "lens_squeeze", 0.0f, lensSqueezeE );
                    setFloatXML( "vertical_aperture", 0.0f, verticalApertureE );

                    techniqueE->mChildren.push_back( filmFitE );
                    techniqueE->mChildren.push_back( filmFitOffsetE );
                    techniqueE->mChildren.push_back( filmOffsetXE );
                    techniqueE->mChildren.push_back( filmOffsetYE );
                    techniqueE->mChildren.push_back( horizontalApertureE );
                    techniqueE->mChildren.push_back( lensSqueezeE );
                    techniqueE->mChildren.push_back( verticalApertureE );

                }
            }
        }

    }

}
