// Copyright(C)SEGA 
#include "Collada/Camera.h"
#include "Collada/Image.h"
#include "Collada/Light.h"
#include "Collada/Effect.h"
#include "Collada/Material.h"
#include "Collada/Geometry.h"
#include "Collada/VisualScene.h"
#include "Collada/SkinController.h"
#include "Collada/MorphController.h"
#include "Collada/Asset.h"
#include "Collada/Collection.h"
#include "Collada/Util.h"
#include "Collada/Animation.h"
#include "Collada/Macro.h"
#include "Collada/Node.h"
#include "Collada/Extra.h"
#include "Collada/ExtraAttribute.h"
#include "Collada/Variant.h"
#include "Collada/XML.h"

#include <iostream>

using namespace std;

namespace Collada {

    // Array猟
    template <typename T>
    T* findArrayElement( vector<T*>& array, const string id )
    {		
        const int num = array.size();
        for( int i=0; i<num; i++ ) {

            if( id == array[ i ]->mId ) {

                return array[ i ];

            }

        }
        return NULL;
    }

    // RXgN^
    Collection::Collection() : mAsset( NULL ), mExtra( NULL )
    {
    }

    // fXgN^
    Collection::~Collection()
    {
        SAFE_DELETE( mAsset );
        SAFE_DELETE( mExtra );

        for( unsigned i=0; i<mCameras.size(); i++ ) {

            SAFE_DELETE( mCameras[ i ] );

        }
        for( unsigned i=0; i<mLights.size(); i++ ) {

            SAFE_DELETE( mLights[ i ] );

        }
        for( unsigned i=0; i<mImages.size(); i++ ) {

            SAFE_DELETE( mImages[ i ] );

        }
        for( unsigned i=0; i<mEffects.size(); i++ ) {

            SAFE_DELETE( mEffects[ i ] );

        }
        for( unsigned i=0; i<mMaterials.size(); i++ ) {

            SAFE_DELETE( mMaterials[ i ] );

        }
        for( unsigned i=0; i<mGeometries.size(); i++ ) {

            SAFE_DELETE( mGeometries[ i ] );

        }
        for( unsigned i=0; i<mVisualScenes.size(); i++ ) {

            SAFE_DELETE( mVisualScenes[ i ] );

        }
        for( unsigned i=0; i<mControllers.size(); i++ ) {

            SAFE_DELETE( mControllers[ i ] );

        }
        for( unsigned i=0; i<mAnimations.size(); i++ ) {

            SAFE_DELETE( mAnimations[ i ] );

        }
    }

    // GtFNg擾
    Effect* Collection::getEffect( const string& id )
    {
        string source;
        Util::sourceToId( id, &source );				
        return findArrayElement( mEffects, source );
    }

    // C[W擾
    Image* Collection::getImage( const string& id )
    {
        string source;
        Util::sourceToId( id, &source );				
        return findArrayElement( mImages, source );				
    }

    // }eA擾
    Material* Collection::getMaterial( const string& id )
    {
        string source;
        Util::sourceToId( id, &source );				
        return findArrayElement( mMaterials, source );				
    }

    // WIg擾
    Geometry* Collection::getGeometry( const string& id )
    {
        string source;
        Util::sourceToId( id, &source );					
        return findArrayElement( mGeometries, source.c_str() );				
    }   

    // XMLɕϊ
    bool Collection::toXML( XMLDocument* doc )
    {
        const char* xmlns = "http://www.collada.org/2005/11/COLLADASchema";

        XMLElement* rootE = new XMLElement();

        rootE->mName = "COLLADA";
        rootE->mAttributes.push_back( new XMLAttribute( "xmlns", xmlns ) );
        rootE->mAttributes.push_back( new XMLAttribute( "version", "1.4.1" ) );

        doc->mRootElement = rootE;

        if( !mAsset ) {

            return false;

        }
        XMLElement* assetE = new XMLElement();
        mAsset->toXML( assetE );
        rootE->mChildren.push_back( assetE );

        // J
        if( mCameras.size() != 0 ) {

            XMLElement* libraryCameraE = new XMLElement();			
            libraryCameraE->mName = "library_cameras";            
            {
                const int cameraNumber = mCameras.size();
                for( int i=0; i<cameraNumber; i++ ) {

                    XMLElement* cameraE = new XMLElement();
                    mCameras[ i ]->toXML( cameraE );
                    libraryCameraE->mChildren.push_back( cameraE );

                }
            }
            rootE->mChildren.push_back( libraryCameraE );

        }

        // Cg
        if( mLights.size() != 0 ) {

            XMLElement* libraryLightE = new XMLElement();
            libraryLightE->mName = "library_lights";            
            {
                const int lightNumber = mLights.size();
                for( int i=0; i<lightNumber; i++ ) {

                    XMLElement* lightE = new XMLElement();
                    mLights[ i ]->toXML( lightE );
                    libraryLightE->mChildren.push_back( lightE );

                }

            }
            rootE->mChildren.push_back( libraryLightE );

        }

        // }eA
        if( mMaterials.size() != 0 ) {

            XMLElement* libraryMaterialE = new XMLElement();
            libraryMaterialE->mName = "library_materials";            
            {
                const int materialNumber = mMaterials.size();
                for( int i=0; i<materialNumber; i++ ) {

                    XMLElement* materialE = new XMLElement();
                    mMaterials[ i ]->toXML( materialE );
                    libraryMaterialE->mChildren.push_back( materialE );

                }

            }
            rootE->mChildren.push_back( libraryMaterialE );

        }

        // GtFNg
        if( mEffects.size() != 0 ) {

            XMLElement* libraryEffectsE = new XMLElement();
            libraryEffectsE->mName = "library_effects";            
            {
                const int effectNumber = mEffects.size();
                for( int i=0; i<effectNumber; i++ ) {

                    XMLElement* effectE = new XMLElement();
                    mEffects[ i ]->toXML( effectE );
                    libraryEffectsE->mChildren.push_back( effectE );

                }
            }
            rootE->mChildren.push_back( libraryEffectsE );

        };

        // C[W
        if( mImages.size() != 0 ) {

            XMLElement* libraryImageE = new XMLElement();
            libraryImageE->mName = "library_images";            
            {
                const int imageNumber = mImages.size();
                for( int i=0; i<imageNumber; i++ ) {

                    XMLElement* imageE = new XMLElement();
                    mImages[ i ]->toXML( imageE );
                    libraryImageE->mChildren.push_back( imageE );

                }
            }
            rootE->mChildren.push_back( libraryImageE );

        }

        // Rg[[
        if( mControllers.size() != 0 ) {

            XMLElement* libraryControllersE = new XMLElement();
            libraryControllersE->mName = "library_controllers";            
            {
                const int controllerNumber = mControllers.size();
                for( int i=0; i<controllerNumber; i++ ) {

                    XMLElement* controllerE = new XMLElement();
                    mControllers[ i ]->toXML( controllerE );
                    libraryControllersE->mChildren.push_back( controllerE );

                }

            }
            rootE->mChildren.push_back( libraryControllersE );

        }

        // Aj[V
        if( mAnimations.size() != 0 ) {

            XMLElement* libraryAnimationsE = new XMLElement();
            libraryAnimationsE->mName = "library_animations";            
            {
                const int animationNumber = mAnimations.size();
                for( int i=0; i<animationNumber; i++ ) {

                    XMLElement* animationE = new XMLElement();
                    mAnimations[ i ]->toXML( animationE );
                    libraryAnimationsE->mChildren.push_back( animationE );

                }
            }
            rootE->mChildren.push_back( libraryAnimationsE );

        }

        // WIg
        if( mGeometries.size() != 0 ) {

            XMLElement* libraryGeometryE = new XMLElement();
            libraryGeometryE->mName = "library_geometries";            
            {
                const int geometryNumber = mGeometries.size();
                for( int i=0; i<geometryNumber; i++ ) {

                    XMLElement* geometryE = new XMLElement();
                    mGeometries[ i ]->toXML( geometryE );
                    libraryGeometryE->mChildren.push_back( geometryE );

                }
            }
            rootE->mChildren.push_back( libraryGeometryE );

        }

        // rWAV[
        if( mVisualScenes.size() != 0 ) {

            XMLElement* libraryVisualSceneE = new XMLElement();
            libraryVisualSceneE->mName = "library_visual_scenes";            
            {
                const int visualSceneNumber = mVisualScenes.size();
                for( int i=0; i<visualSceneNumber; i++ ) {

                    XMLElement* visualSceneE = new XMLElement();
                    mVisualScenes[ i ]->toXML( visualSceneE );
                    libraryVisualSceneE->mChildren.push_back( visualSceneE );

                }
            }
            rootE->mChildren.push_back( libraryVisualSceneE );

        }

        // V[
        {
            XMLElement* sceneE = new XMLElement();
            sceneE->mName = "scene";			
            {
                XMLElement* instanceVisualSceneE = new XMLElement();

                instanceVisualSceneE->mName = "instance_visual_scene";				
                instanceVisualSceneE->mAttributes.push_back( new XMLAttribute( "url", mScene.instanceVisualScene.mURL ) );

                sceneE->mChildren.push_back( instanceVisualSceneE );
            }
            rootE->mChildren.push_back( sceneE );
        }

        // Extra
        if( mExtra ) {

            XMLElement* extraE = new XMLElement();
            mExtra->toXML( extraE );
            rootE->mChildren.push_back( extraE );

        }

        return true;
    }

    // o
    bool Collection::save( const string& filename ) {

        XMLDocument* doc = new XMLDocument();

        if( !toXML( doc ) ) {

            delete doc;
            doc = NULL;

            return false;

        }

        //o
        if( !doc->save( filename ) ) {

            cout << "err: can not write file = " << filename << endl;
            return false;

        }

        return true;
    }

}
