// Copyright(C)SEGA 
#include "Collada/ExtraAttribute.h"
#include "Collada/Variant.h"
#include "Collada/Struct.h"
#include "Collada/XML.h"

#include <string>
using namespace std;

namespace Collada {

    // RXgN^
    ExtraAttribute::ExtraAttribute() : mValue(NULL)
    {
    }

    // Rs[RXgN^
    ExtraAttribute::ExtraAttribute( const ExtraAttribute& ea ) : mValue(NULL), Resource( ea )
    {
        // value
        SAFE_DELETE( mValue );
        if( ea.mValue ) {

            mValue = new Variant();
            *mValue = *ea.mValue;

        }

        // attribute
        mAttributes.clear();
        mAttributes = ea.mAttributes;
    }

    // fXgN^
    ExtraAttribute::~ExtraAttribute()
    {
        SAFE_DELETE( mValue );
    }

    // Agr[g擾
    Attribute* ExtraAttribute::getAttribute( const string& id )
    {
        const int attributeNumber = mAttributes.size();
        for( int i=0; i<attributeNumber; i++ ) {

            if( mAttributes[ i ].mName == id ) {

                return &mAttributes[ i ];

            }

        }
        return NULL;
    }

    // XMLݒ
    void ExtraAttribute::toXML( XMLElement* e )
    {
        // l݂͑ȂƂ
        if( mValue ) {

            switch( mValue->mVariantType ) {

            case Variant::VariantType_FLOAT:
                setFloatXML( mId, mValue->mFloatValue, e );
                break;

            case Variant::VariantType_INT:
                setIntXML( mId, mValue->mIntValue, e );
                break;

            case Variant::VariantType_BOOL:
                setBoolXML( mId, mValue->mBoolValue, e );
                break;

            case Variant::VariantType_STRING:
                setStringXML( mId, mValue->mStringValue, e );
                break;

            case Variant::VariantType_COLOR:
                setColorXML( mId, mValue->mColorValue, e );
                break;

            }

        }
        else {

            e->mName = mId;

        }

        const int attributeNumber = mAttributes.size();
        for( int i=0; i<attributeNumber; i++ ) {

            e->mAttributes.push_back( new XMLAttribute( mAttributes[ i ].mName, mAttributes[ i ].mValue ) );

        }
    }

    // operator==
    bool ExtraAttribute::operator==( const ExtraAttribute& ea ) const
    {
        // Agr[gʃm
        if( mAttributes.size() != ea.mAttributes.size() ) {

            return false;

        }

        // Agr[głʃm
        const int attributeNumber = mAttributes.size();
        for( int i=0; i<attributeNumber; i++ ) {

            if( mAttributes[ i ] != ea.mAttributes[ i ] ) {

                return false;

            }

        }

        // ǂNULLłȂƂrł
        if( mValue != NULL && ea.mValue != NULL ) {

            if( *mValue != *ea.mValue ) {

                return false;

            }
        }
        // ǂ炩ЕNULL̂Ƃ́uႤv
        else if( ( mValue == NULL && ea.mValue != NULL ) ||
            ( mValue != NULL && ea.mValue == NULL ) ) {

                return false;

        }

        return true;
    }

    // operator!=
    bool ExtraAttribute::operator!=( const ExtraAttribute& ea ) const
    {
        return ( *this == ea ) ? false : true;
    }

}
