// Copyright(C)SEGA 
#include "Collada/Geometry.h"
#include "Collada/Macro.h"
#include "Collada/StructVector.h"
#include "Collada/Vertices.h"
#include "Collada/PolyList.h"
#include "Collada/Triangles.h"
#include "Collada/Util.h"
#include "Collada/Extra.h"
#include "Collada/XML.h"

#include <string>
using namespace std;

namespace Collada {

    // Array猟
    template <typename T>
    T* findArrayElement( vector<T*>& array, const string& id )
    {
        const int num = array.size();
        for( int i=0; i<num; i++ ) {

            if( id == array[ i ]->mId ) {

                return array[ i ];

            }

        }
        return NULL;
    }

    // fXgN^
    Geometry::~Geometry()
    {
        const int posNumber = mPositions.size();
        for( int i=0; i<posNumber; i++ ) {

            SAFE_DELETE( mPositions[ i ] );

        }
        const int nrmNumber = this->mNormals.size();
        for( int i=0; i<nrmNumber; i++ ) {

            SAFE_DELETE( mNormals[ i ] );

        }
        const int colNumber = mColors.size();
        for( int i=0; i<colNumber; i++ ) {

            SAFE_DELETE( mColors[ i ] );

        }
        const int geoTangentNumber = mGeoTangents.size();
        for( int i=0; i<geoTangentNumber; i++ ) {

            SAFE_DELETE( mGeoTangents[ i ] );

        }
        const int geoBinormalNumber = mGeoBinormals.size();
        for( int i=0; i<geoBinormalNumber; i++ ) {

            SAFE_DELETE( mGeoBinormals[ i ] );

        }
        const int uvNumber = mUVs.size();
        for( int i=0; i<uvNumber; i++ ) {

            SAFE_DELETE( mUVs[ i ] );

        }
        const int tanNumber = mTangents.size();
        for( int i=0; i<tanNumber; i++ ) {

            SAFE_DELETE( mTangents[ i ] );

        }
        const int binNumber = mBinormals.size();
        for( int i=0; i<binNumber; i++ ) {

            SAFE_DELETE( mBinormals[ i ] );

        }
        const int vertNumber = mVertices.size();
        for( int i=0; i<vertNumber; i++ ) {

            SAFE_DELETE( mVertices[ i ] );

        }
        const int polyListNumber = mPolyLists.size();
        for( int i=0; i<polyListNumber; i++ ) {

            SAFE_DELETE( mPolyLists[ i ] );

        }
        const int triNumber = mTriangles.size();
        for( int i=0; i<triNumber; i++ ) {

            SAFE_DELETE( mTriangles[ i ] );

        }

    }

    // source擾(id)
    StructVector* Geometry::getSource( const string& id )
    {
        StructVector* vec = NULL;

        vec = getPositionByVertices( id );
        if( vec ) { return vec; }

        vec = getPosition( id );
        if( vec ) { return vec; }

        vec = getNormal( id );
        if( vec ) { return vec; }

        vec = getColor( id );
        if( vec ) { return vec; }

        vec = getGeoTangent( id );
        if( vec ) { return vec; }

        vec = getGeoBinormal( id );
        if( vec ) { return vec; }

        vec = getUV( id );
        if( vec ) { return vec; }

        vec = getTangent( id );
        if( vec ) { return vec; }

        vec = getBinormal( id );
        if( vec ) { return vec; }

        return vec;
    }

    // source擾(index)
    StructVector* Geometry::getSource( const int index )
    {
        int count = 0;

        const int posNumber = mPositions.size();
        for( int i=0; i<posNumber; i++ ) {

            if( count == index ) {

                return mPositions[ i ];

            }
            count++;

        }
        const int nrmNumber = mNormals.size();
        for( int i=0; i<nrmNumber; i++ ) {

            if( count == index ) {

                return mNormals[ i ];

            }
            count++;

        }
        const int colNumber = mColors.size();
        for( int i=0; i<colNumber; i++ ) {

            if( count == index ) {

                return mColors[ i ];

            }
            count++;

        }
        const int geoTanNumber = mGeoTangents.size();
        for( int i=0; i<geoTanNumber; i++ ) {

            if( count == index ) {

                return mGeoTangents[ i ];

            }
            count++;

        }
        const int geoBinNumber = mGeoBinormals.size();
        for( int i=0; i<geoBinNumber; i++ ) {

            if( count == index ) {

                return mGeoBinormals[ i ];

            }
            count++;

        }
        const int uvNum = mUVs.size();
        for( int i=0; i<uvNum; i++ ) {

            if( count == index ) {

                return mUVs[ i ];

            }
            count++;

        }
        const int tanNumber = mTangents.size();
        for( int i=0; i<tanNumber; i++ ) {

            if( count == index ) {

                return mTangents[ i ];

            }
            count++;

        }
        const int binNumber = mBinormals.size();
        for( int i=0; i<binNumber; i++ ) {

            if( count == index ) {

                return mBinormals[ i ];

            }
            count++;

        }

        return NULL;
    }

    // source̐擾
    int Geometry::getSourceNumber() const
    {
        int count = 0;

        count += mPositions.size();
        count += mNormals.size();
        count += mColors.size();
        count += mGeoTangents.size();
        count += mGeoBinormals.size();
        count += mUVs.size();
        count += mTangents.size();
        count += mBinormals.size();

        return count;
    }

    // ʒu擾
    Float3Vector* Geometry::getPosition( const string& id )
    {
        return findArrayElement( mPositions, id );
    }

    // ʒu擾
    Float3Vector* Geometry::getPositionByVertices( const string& id )
    {
        Vertices* vert = getVertices( id );
        if( !vert ) {

            return NULL;

        }
        string source;
        Util::sourceToId( vert->mInput->mSource, &source );

        return getPosition( source );
    }

    // @擾
    Float3Vector* Geometry::getNormal( const string& id )
    {
        return findArrayElement( mNormals, id );
    }

    // J[擾
    ColorVector* Geometry::getColor( const string& id )
    {
        return findArrayElement( mColors, id );
    }

    // GeoTangent擾
    Float3Vector* Geometry::getGeoTangent( const string& id )
    {
        return findArrayElement( mGeoTangents, id );
    }

    // GeoBinormal擾
    Float3Vector* Geometry::getGeoBinormal( const string& id )
    {
        return findArrayElement( mGeoBinormals, id );
    }

    // UV擾
    Float2Vector* Geometry::getUV( const string& id )
    {
        return findArrayElement( mUVs, id );
    }

    // Tangent擾
    Float3Vector* Geometry::getTangent( const string& id )
    {
        return findArrayElement( mTangents, id );
    }

    // Binormal擾
    Float3Vector* Geometry::getBinormal( const string& id )
    {
        return findArrayElement( mBinormals, id );
    }

    // _(Vertices)擾
    Vertices* Geometry::getVertices( const string& id )
    {
        return findArrayElement( mVertices, id );
    }

    // XMLݒ
    void Geometry::toXML( XMLElement* e )
    {
        e->mName = "geometry";

        e->mAttributes.push_back( new XMLAttribute( "id", mId ) );
        e->mAttributes.push_back( new XMLAttribute( "name", mName ) );
        {
            // mesh
            {
                XMLElement* meshE = new XMLElement();
                {
                    meshE->mName = "mesh";

                    // ʒu
                    const int posNumber = mPositions.size();
                    for( int i=0; i<posNumber; i++ ) {

                        XMLElement* positionE = new XMLElement();
                        mPositions[ i ]->toXML( positionE );
                        meshE->mChildren.push_back( positionE );

                    }

                    // @
                    const int norNumber = mNormals.size();
                    for( int i=0; i<norNumber; i++ ) {

                        XMLElement* normalE = new XMLElement();
                        mNormals[ i ]->toXML( normalE );
                        meshE->mChildren.push_back( normalE );

                    }

                    // J[
                    const int colNumber = mColors.size();
                    for( int i=0; i<colNumber; i++ ) {

                        XMLElement* colorE = new XMLElement();
                        mColors[ i ]->toXML( colorE );
                        meshE->mChildren.push_back( colorE );

                    }

                    // ڐo(Geometryt)
                    const int geoTangentNumber = mGeoTangents.size();
                    for( int i=0; i<geoTangentNumber; i++ ) {

                        XMLElement* geoTangentE = new XMLElement();
                        mGeoTangents[ i ]->toXML( geoTangentE );
                        meshE->mChildren.push_back( geoTangentE );

                    }

                    // ]@o(Geometryt)
                    const int geoBinormalNumber = mGeoBinormals.size();
                    for( int i=0; i<geoBinormalNumber; i++ ) {

                        XMLElement* geoBinormalE = new XMLElement();
                        mGeoBinormals[ i ]->toXML( geoBinormalE );
                        meshE->mChildren.push_back( geoBinormalE );

                    }

                    // UVo()
                    const int uvNumber = mUVs.size();
                    for( int i=0; i<uvNumber; i++ ) {

                        XMLElement* uvE = new XMLElement();
                        mUVs[ i ]->toXML( uvE );
                        meshE->mChildren.push_back( uvE );

                    }

                    // ڐo(UVP)
                    const int tangentNumber = mTangents.size();
                    for( int i=0; i<tangentNumber; i++ ) {

                        XMLElement* tangentE = new XMLElement();
                        mTangents[ i ]->toXML( tangentE );
                        meshE->mChildren.push_back( tangentE );

                    }

                    // ]@o(UVP)
                    const int binormalNumber = mBinormals.size();
                    for( int i=0; i<binormalNumber; i++ ) {

                        XMLElement* binormalE = new XMLElement();
                        mBinormals[ i ]->toXML( binormalE );
                        meshE->mChildren.push_back( binormalE );

                    }

                    // Vertices
                    const int verticesNumber = mVertices.size();
                    for( int i=0; i<verticesNumber; i++ ) {

                        XMLElement* verticesE = new XMLElement();
                        {
                            verticesE->mName = "vertices";
                            verticesE->mAttributes.push_back( new XMLAttribute( "id", mVertices[ i ]->mId ) );
                            verticesE->mAttributes.push_back( new XMLAttribute( "name", mVertices[ i ]->mName ) );
                        }
                        meshE->mChildren.push_back( verticesE );

                        {
                            XMLElement* inputE = new XMLElement();
                            {
                                inputE->mName = "input";
                                inputE->mAttributes.push_back( new XMLAttribute( "semantic", Util::getSemanticStr( mVertices[ i ]->mInput->mSemantic ) ) );
                                inputE->mAttributes.push_back( new XMLAttribute( "source", mVertices[ i ]->mInput->mSource ) );
                            }
                            verticesE->mChildren.push_back( inputE );
                        }

                    }

                    // polylist
                    const int polyListNumber = mPolyLists.size();
                    for( int i=0; i<polyListNumber; i++ ) {

                        XMLElement* polyListE = new XMLElement();
                        mPolyLists[ i ]->toXML( polyListE );
                        meshE->mChildren.push_back( polyListE );

                    }

                    // triangles
                    const int trianglesNumber = mTriangles.size();
                    for( int i=0; i<trianglesNumber; i++ ) {

                        XMLElement* trianglesE = new XMLElement();
                        mTriangles[ i ]->toXML( trianglesE );
                        meshE->mChildren.push_back( trianglesE );

                    }

                }
                e->mChildren.push_back( meshE );

            }

            // extra
            if( mExtra ) {

                XMLElement* extraE = new XMLElement();
                mExtra->toXML( extraE );
                e->mChildren.push_back( extraE );

            }

        }

    }

}
