// Copyright(C)SEGA 
#include "Collada/Profile.h"
#include "Collada/Technique.h"
#include "Collada/Struct.h"
#include "Collada/Util.h"
#include "Collada/Technique.h"
#include "Collada/ColorOrTextureOrFloat.h"
#include "Collada/XML.h"

#include <string>
using namespace std;

namespace Collada {

    // RXgN^
    Profile::Profile() : mTechnique(NULL)
    {
    }

    // fXgN^
    Profile::~Profile()
    {
        SAFE_DELETE( mTechnique );
    }

    // Surface擾
    Surface Profile::getSurface( const string& sid ) const
    {
        const int surfaceNumber = mSurfaces.size();
        for( int i=0; i<surfaceNumber; i++ ) {

            if( sid == mSurfaces[ i ].mSId ) {

                return mSurfaces[ i ];

            }

        }
        return Surface();
    }

    // Sampler2D擾
    Sampler2D Profile::getSampler2D( const string& sid ) const
    {
        const int sampler2DNumber = mSampler2Ds.size();
        for( int i=0; i<sampler2DNumber; i++ ) {

            if( sid == mSampler2Ds[ i ].mSId ) {

                return mSampler2Ds[ i ];

            }
        }
        return Sampler2D();
    }


    // XMLݒ
    void Profile::toXML( XMLElement* e )
    {
        e->mName = mName;

        // new_param

        // surface
        const int surfaceNumber = mSurfaces.size();
        for( int i=0; i<surfaceNumber; i++ ) {

            XMLElement* newParamE = new XMLElement();
            newParamE->mName = "newparam";
            newParamE->mAttributes.push_back( new XMLAttribute( "sid", mSurfaces[ i ].mSId ) );            
            {
                XMLElement* surfaceE = new XMLElement();
                surfaceE->mName = "surface";                
                surfaceE->mAttributes.push_back( new XMLAttribute( "type", mSurfaces[ i ].mType ) );                
                {
                    XMLElement* initFromE = new XMLElement();
                    setStringXML( "init_from", mSurfaces[ i ].mInitFrom, initFromE );
                    surfaceE->mChildren.push_back( initFromE );

                    XMLElement* formatE = new XMLElement();
                    setStringXML( "format", mSurfaces[ i ].mFormat, formatE );
                    surfaceE->mChildren.push_back( formatE );
                }
                newParamE->mChildren.push_back( surfaceE );
            }
            e->mChildren.push_back( newParamE );

        }

        // sampler2D
        const int sampler2DNumber = mSampler2Ds.size();
        for( int i=0; i<sampler2DNumber; i++ ) {

            XMLElement* newParamE = new XMLElement();
            newParamE->mName = "newparam";            
            newParamE->mAttributes.push_back( new XMLAttribute( "sid", mSampler2Ds[ i ].mSId ) );            
            {
                XMLElement* sampler2DE = new XMLElement();
                sampler2DE->mName = "sampler2D";                
                {
                    XMLElement* sourceE = new XMLElement();
                    setStringXML( "source", mSampler2Ds[ i ].mSource, sourceE );
                    sampler2DE->mChildren.push_back( sourceE );
                }
                newParamE->mChildren.push_back( sampler2DE );
            }
            e->mChildren.push_back( newParamE );

        }

        // technique
        XMLElement* techniqueE = new XMLElement();
        techniqueE->mName = "technique";        
        techniqueE->mAttributes.push_back( new XMLAttribute( "sid", "common" ) );
        if( mTechnique ) {

            XMLElement* techniqueInfoE = new XMLElement();			
            mTechnique->toXML( techniqueInfoE );
            techniqueE->mChildren.push_back( techniqueInfoE );

        }
        e->mChildren.push_back( techniqueE );

    }

}
