// Copyright(C)SEGA 
#include "Collada/Resource.h"
#include "Collada/Macro.h"
#include "Collada/XML.h"

#include <sstream>

using namespace std;

namespace Collada {

    // XMLݒ(int)
    void Resource::setIntXML( const string& name, const int v, XMLElement* e )
    {
        ostringstream ss;
        ss << v;

        e->mName = name;
        e->mValue = ss.str();
    }

    // XMLݒ(bool)
    void Resource::setBoolXML( const string& name, const bool v, XMLElement* e )
    {
        e->mName = name;

        // bool͑啶"TRUE","FALSE"Ŋi[
        if( v ) {

            e->mValue = "TRUE";

        }
        else {

            e->mValue = "FALSE";

        }
    }

    // XMLݒ(float)
    void Resource::setFloatXML( const string& name, const float v, XMLElement* e )
    {
        ostringstream ss;
        ss << v;

        e->mName = name;

        string content = ss.str();        

        // '.'Ō'.0't
        if( string::npos == content.find( "." ) ) {

            ss << ".0";
            content = ss.str();

        }

        e->mValue = content;
    }

    // XMLݒ(Color)
    void Resource::setColorXML( const string& name, const Color& v, XMLElement* e )
    {
        ostringstream ss;

        ss << v.r << " " << v.g << " " << v.b << " " << v.a;

        e->mName = name;
        e->mValue = ss.str();
    }

    // XMLݒ(string)
    void Resource::setStringXML( const string& name, const string& v, XMLElement* e )
    {
        e->mName = name;
        e->mValue = v;
    }

    // XMLݒ(Matrix)
    void Resource::setMatrixXML( const string& name, const Matrix& v, XMLElement* e )
    {
        ostringstream ss;

        for( unsigned i=0; i<16; i++ ) {

            ss << v.m[ i ];

            if( i+1 != 16 ) {

                ss << " ";

            }

        }

        e->mName = name;
        e->mValue = ss.str();
    }

}
