// Copyright(C)SEGA 
#include "Collada/SkinController.h"
#include "Collada/Macro.h"
#include "Collada/StructVector.h"
#include "Collada/Util.h"
#include "Collada/VertexWeights.h"
#include "Collada/XML.h"

namespace Collada {

    // RXgN^
    SkinController::SkinController() :
mInvBindMatrix( NULL ),
mVertexWeights( NULL )
{
    mType = Type_SKIN;
}

// fXgN^
SkinController::~SkinController()
{
    SAFE_DELETE( mInvBindMatrix );
    SAFE_DELETE( mVertexWeights );
}

// XMLݒ
void SkinController::toXMLDetail( XMLElement* e )
{
    e->mName = "skin";		
    e->mAttributes.push_back( new XMLAttribute( "source", mSource ) );

    {
        // VFCṽ}gbNX
        XMLElement* bindShapeMatrix = new XMLElement();
        setMatrixXML( "bind_shape_matrix", mBindShapeMatrix, bindShapeMatrix );
        e->mChildren.push_back( bindShapeMatrix );

        // WCg
        XMLElement* jointE = new XMLElement();
        mTarget->toXML( jointE );
        e->mChildren.push_back( jointE );

        // InvBindMatrix
        XMLElement* invBindMatrixE = new XMLElement();
        mInvBindMatrix->toXML( invBindMatrixE );
        e->mChildren.push_back( invBindMatrixE );

        // EFCg
        XMLElement* weightE = new XMLElement();
        mWeight->toXML( weightE );
        e->mChildren.push_back( weightE );

        // target
        if( mInputs.size() != 0 ) {

            XMLElement* jointsE = new XMLElement();
            jointsE->mName = "joints";
            {
                const int inputNumber = mInputs.size();
                for( int i=0; i<inputNumber; i++ ) {

                    XMLElement* inputE = new XMLElement();

                    inputE->mName = "input";
                    inputE->mAttributes.push_back( new XMLAttribute( "semantic", Util::getSemanticStr( mInputs[ i ].mSemantic ) ) );
                    inputE->mAttributes.push_back( new XMLAttribute( "source", mInputs[ i ].mSource ) );

                    jointsE->mChildren.push_back( inputE );

                }

            }
            e->mChildren.push_back( jointsE );

        }

        // vertexWeights
        if( mVertexWeights ) {

            XMLElement* vertexWeightE = new XMLElement();
            mVertexWeights->toXML( vertexWeightE );
            e->mChildren.push_back( vertexWeightE );

        }

    }

}

}

