// Copyright(C)SEGA 
#include "Collada/StructVector.h"
#include "Collada/Macro.h"
#include "Collada/Accessor.h"
#include "Collada/Util.h"
#include "Collada/XML.h"

#include <sstream>

using namespace std;

namespace Collada {

    // fXgN^
    StructVector::~StructVector()
    {
        SAFE_DELETE( mAccessor );
    }

    // XMLݒ
    void StructVector::toXML( XMLElement* e )
    {
        e->mName = "source";
        e->mAttributes.push_back( new XMLAttribute( "id", mId ) );
        {
            string arrayString;
            arrayToString( &arrayString );

            XMLElement* arrayE = new XMLElement();
            {
                arrayE->mName = mArrayTag;            
                arrayE->mAttributes.push_back( new XMLAttribute( "id", mArrayName ) );
                arrayE->mAttributes.push_back( new XMLAttribute( "count", getCount() ) );
                arrayE->mValue = arrayString;
            }
            e->mChildren.push_back( arrayE );

            XMLElement* techniqueCommonE = new XMLElement();
            {
                techniqueCommonE->mName = "technique_common";

                if( mAccessor ) {

                    XMLElement* accessorE = new XMLElement();
                    mAccessor->toXML( accessorE );
                    techniqueCommonE->mChildren.push_back( accessorE );

                }
            }
            e->mChildren.push_back( techniqueCommonE );

        }
    }

    // RXgN^
    FloatVector::FloatVector()
    {
    }

    // z𕶎ɕϊ
    void FloatVector::arrayToString( std::string* ret )
    {
        ostringstream ss;

        for( unsigned i=0; i<mElement.size(); i++ ) {

            ss << mElement[ i ] << " ";

        }

        *ret = ss.str();
    }

    // z̗vf擾(float̑)
    int FloatVector::getCount() const
    {
        return mElement.size();
    }

    // RXgN^
    Float2Vector::Float2Vector()
    {
    }

    // z̗vf擾(float̑)
    int Float2Vector::getCount() const
    {
        return mElement.size() * 2;
    }

    // z𕶎ɕϊ
    void Float2Vector::arrayToString( std::string* ret )
    {
        ostringstream ss;

        for( unsigned i=0; i<mElement.size(); i++ ) {

            ss << mElement[ i ].x << " " << mElement[ i ].y << " ";

        }
        *ret = ss.str();
    }

    // RXgN^
    Float3Vector::Float3Vector()
    {
    }

    // z̗vf擾(float̑)
    int Float3Vector::getCount() const
    {
        return mElement.size() * 3;
    }

    // z𕶎ɕϊ
    void Float3Vector::arrayToString( std::string* ret )
    {
        ostringstream ss;

        for( unsigned i=0; i<mElement.size(); i++ ) {

            ss << mElement[ i ].x << " " << mElement[ i ].y << " " << mElement[ i ].z << " ";

        }

        *ret = ss.str();
    }

    // RXgN^
    ColorVector::ColorVector()
    {
    }

    // z̗vf擾(float̑)
    int ColorVector::getCount() const
    {
        return mElement.size() * 4;
    }

    // z𕶎ɕϊ
    void ColorVector::arrayToString( std::string* ret )
    {
        ostringstream ss;

        for( unsigned i=0; i<mElement.size(); i++ ) {

            ss << mElement[ i ].r << " " << mElement[ i ].g << " " << mElement[ i ].b << " " << mElement[ i ].a << " ";

        }

        *ret = ss.str();
    }

    // RXgN^
    IntVector::IntVector()
    {
    }

    // z̗vf擾(int̑)
    int IntVector::getCount() const
    {
        return mElement.size();
    }

    // z𕶎ɕϊ
    void IntVector::arrayToString( std::string* ret )
    {
        ostringstream ss;

        for( unsigned i=0; i<mElement.size(); i++ ) {

            ss << mElement[ i ] << " ";

        }

        *ret = ss.str();
    }

    // RXgN^
    StringVector::StringVector()
    {
    }

    // z̗vf擾(string̑)
    int StringVector::getCount() const
    {
        return mElement.size();
    }

    // z𕶎ɕϊ
    void StringVector::arrayToString( std::string* ret )
    {
        ostringstream ss;

        for( unsigned i=0; i<mElement.size(); i++ ) {

            ss << mElement[ i ] << " ";

        }

        *ret = ss.str();
    }

    // RXgN^
    MatrixVector::MatrixVector()
    {
    }

    // z̗vf擾(float̑)
    int MatrixVector::getCount() const
    {
        return mElement.size() * 16;
    }

    // z𕶎ɕϊ
    void MatrixVector::arrayToString( std::string* ret )
    {
        ostringstream ss;

        for( unsigned i=0; i<mElement.size(); i++ ) {

            for( int m=0; m<16; m++ ) {

                ss << mElement[ i ].m[ m ] << " ";

            }

        }

        *ret = ss.str();
    }

}
