// Copyright(C)SEGA 
#include "Collada/Technique.h"
#include "Collada/Macro.h"
#include "Collada/Texture.h"
#include "Collada/ColorOrTextureOrFloat.h"
#include "Collada/XML.h"
#include "Collada/Extra.h"

#include <string>

using namespace std;

namespace Collada {

    // RXgN^
    Technique::Technique()
    {
    }    

    // fXgN^
    Technique::~Technique()
    {
        const int num = mColorOrTextureOrFloats.size();
        for( int i=0; i<num; i++ ) {

            SAFE_DELETE( mColorOrTextureOrFloats[ i ] );

        }
        mColorOrTextureOrFloats.clear();

        const int numExtra = mColorOrTextureOrFloatsExtra.size();
        for( int i=0; i<numExtra; i++ ) {

            SAFE_DELETE( mColorOrTextureOrFloatsExtra[ i ] );

        }
        mColorOrTextureOrFloatsExtra.clear();
    }

    // eNX`ǉ
    void Technique::addTexture( const string& attrName, Texture* texture )
    {
        ColorOrTextureOrFloat* ctf = new ColorOrTextureOrFloat();

        ctf->mType = ColorOrTextureOrFloat::Type_TEXTURE;
        ctf->mName = attrName;
        ctf->mTexture = texture;

        if( !isAcceptable( attrName ) ) {

            addColorOrTextureOrFloatExtra( ctf );

        }
        else {

            addColorOrTextureOrFloat( ctf );

        }
    }

    // diffuseȂǂɃJ[ݒ
    void Technique::addColor( const std::string& attrName, const Color& color )
    {
        if( !isAcceptable( attrName ) ) {
            return;
        }

        // ɓTexture݂H
        const int num = mColorOrTextureOrFloats.size();
        bool find = false;
        Texture* texture = NULL;
        for( int i=0; i<num; i++ ) {

            if( mColorOrTextureOrFloats[ i ]->mName == attrName ) {

                find = true;
                if( mColorOrTextureOrFloats[ i ]->mType == ColorOrTextureOrFloat::Type_TEXTURE ) {

                    texture = mColorOrTextureOrFloats[ i ]->mTexture;

                }
                break;

            }

        }

        // ̗vf݂Ȃ΁Â܂ܒǉ
        if( !find ) {

            ColorOrTextureOrFloat* ctf = new ColorOrTextureOrFloat();

            ctf->mType = ColorOrTextureOrFloat::Type_COLOR;
            ctf->mName = attrName;
            ctf->mColor = color;

            addColorOrTextureOrFloat( ctf );

        }
        // ̃eNX`AExtraɒǉ
        else if( texture ) {

            if( texture->mExtra ) {

                texture->mExtra->addExtraAttributeColor( "color", color );

            }

        }
        // ȊȌꍇ͖ĩJ[ꍇj
    }

    // J[ǉ(Attributet)
    void Technique::addColor( const std::string& attrName, const Color& color, const Attribute& attribute )
    {
        if( !isAcceptable( attrName ) ) {
            return;
        }

        ColorOrTextureOrFloat* ctf = new ColorOrTextureOrFloat();

        ctf->mType = ColorOrTextureOrFloat::Type_COLOR;
        ctf->mName = attrName;
        ctf->mColor = color;

        // Agr[gǉ
        ctf->mAttribute.push_back( attribute );

        addColorOrTextureOrFloat( ctf );
    }

    // shininessȂǂfloatlݒ
    void Technique::addFloat( const std::string& attrName, const float value )
    {
        if( !isAcceptable( attrName ) ) {

            return;

        }

        ColorOrTextureOrFloat* ctf = new ColorOrTextureOrFloat();
        ctf->mType = ColorOrTextureOrFloat::Type_FLOAT;
        ctf->mName = attrName;
        ctf->mFloat = value;

        addColorOrTextureOrFloat( ctf );
    }

    // ColorOrTextureOrFloat擾
    ColorOrTextureOrFloat* Technique::colorOrTextureOrFloat( const string& name )
    {
        const int num = mColorOrTextureOrFloats.size();
        for( int i=0; i<num; i++ ) {

            string paramName( mColorOrTextureOrFloats[ i ]->mName );

            if( paramName == name ) {

                return	mColorOrTextureOrFloats[ i ];

            }

        }
        return NULL;
    }

    // XMLݒ
    void Technique::toXML( XMLElement* e )
    {
        e->mName = mName;        

        const int colorOrTextureOrFloatNumber = mColorOrTextureOrFloats.size();
        for( int i=0; i<colorOrTextureOrFloatNumber; i++ ) {

            XMLElement* colorOrTextureOrFloatE = new XMLElement();
            mColorOrTextureOrFloats[ i ]->toXML( colorOrTextureOrFloatE );
            e->mChildren.push_back( colorOrTextureOrFloatE );

        }

        // ExtraɓĂ镪<Extraɏo>
        if( mColorOrTextureOrFloatsExtra.size() > 0 ) {

            XMLElement* extraE = new XMLElement();
            extraE->mName = "Extra";
            e->mChildren.push_back( extraE );

            XMLElement* techniqueE = new XMLElement();
            techniqueE->mName = "technique";
            techniqueE->mAttributes.push_back( new XMLAttribute( "profile", "SICOLLADA_EXPORTER" ) );
            extraE->mChildren.push_back( techniqueE );

            const int num = mColorOrTextureOrFloatsExtra.size();
            for( int i=0; i<num; i++ ) {

                XMLElement* colorOrTextureOrFloatE = new XMLElement();
                mColorOrTextureOrFloatsExtra[ i ]->toXML( colorOrTextureOrFloatE );
                techniqueE->mChildren.push_back( colorOrTextureOrFloatE );

            }

        }
    }

    // ColorOrTextureOrFloatǉ
    bool Technique::addColorOrTextureOrFloat( ColorOrTextureOrFloat* ctf )
    {
        mColorOrTextureOrFloats.push_back( ctf );

        return true;
    }

    // ColorOrTextureOrFloatǉ(Extraցj	
    bool Technique::addColorOrTextureOrFloatExtra( ColorOrTextureOrFloat* ctf )
    {
        mColorOrTextureOrFloatsExtra.push_back( ctf );

        return true;
    }

    // RXgN^
    Constant::Constant()
    {
        mName = "constant";
    }

    // t\ȑ`FbN
    bool Constant::isAcceptable( const std::string& attrName )
    {
        std::string attrNameStr( attrName );

        if( attrNameStr == "emission" ||
            attrNameStr == "reflective" ||
            attrNameStr == "reflectivity" ||
            attrNameStr == "transparent" ||
            attrNameStr == "transparency" ||
            attrNameStr == "indexOfRefraction" ) {

                return true;

        }

        return false;
    }

    // RXgN^
    Lambert::Lambert()
    {
        mName = "lambert";
    }

    // t\ȑ`FbN
    bool Lambert::isAcceptable( const std::string& attrName )
    {
        std::string attrNameStr( attrName );

        if( attrNameStr == "emission" ||
            attrNameStr == "ambient" ||
            attrNameStr == "diffuse" ||
            attrNameStr == "reflective" ||
            attrNameStr == "reflectivity" ||
            attrNameStr == "transparent" ||
            attrNameStr == "transparency" ||
            attrNameStr == "indexOfRefraction" ) {

                return true;

        }

        return false;
    }

    // RXgN^
    Phong::Phong()
    {
        mName = "phong";
    }

    // t\ȑ`FbN
    bool Phong::isAcceptable( const std::string& attrName )
    {
        std::string attrNameStr( attrName );		

        if( attrNameStr == "emission" ||
            attrNameStr == "ambient" ||
            attrNameStr == "diffuse" ||
            attrNameStr == "specular" ||
            attrNameStr == "shininess" ||
            attrNameStr == "reflective" ||
            attrNameStr == "reflectivity" ||
            attrNameStr == "transparent" ||
            attrNameStr == "transparency" ||
            attrNameStr == "indexOfRefraction" ) {

                return true;

        }

        return false;
    }

    // RXgN^
    Blinn::Blinn()
    {
        mName = "blinn";
    }

    // t\ȑ`FbN
    bool Blinn::isAcceptable( const std::string& attrName )
    {
        std::string attrNameStr( attrName );		

        if( attrNameStr == "emission" ||
            attrNameStr == "ambient" ||
            attrNameStr == "diffuse" ||
            attrNameStr == "specular" ||
            attrNameStr == "shininess" ||
            attrNameStr == "reflective" ||
            attrNameStr == "reflectivity" ||
            attrNameStr == "transparent" ||
            attrNameStr == "transparency" ||
            attrNameStr == "indexOfRefraction" ) {

                return true;

        }

        return false;
    }
}
