// Copyright(C)SEGA 
#include "Collada/Camera.h"
#include "Collada/Image.h"
#include "Collada/Light.h"
#include "Collada/Effect.h"
#include "Collada/Material.h"
#include "Collada/Geometry.h"
#include "Collada/VisualScene.h"
#include "Collada/Controller.h"
#include "Collada/Asset.h"
#include "Collada/Collection.h"
#include "Collada/Util.h"
#include "Collada/Animation.h"
#include "Collada/Macro.h"
#include "Collada/Test.h"
#include "Collada/Node.h"
#include "Collada/Vertices.h"
#include "Collada/PolyList.h"
#include "Collada/Triangles.h"
#include "Collada/StructVector.h"
#include "Collada/Profile.h"
#include "Collada/Technique.h"
#include "Collada/Texture.h"
#include "Collada/Struct.h"
#include "Collada/VertexWeights.h"
#include "Collada/ExtraAttribute.h"
#include "Collada/Extra.h"
#include "Collada/Variant.h"
#include "Collada/InstanceGeometry.h"
#include "Collada/ColorOrTextureOrFloat.h"
#include "Collada/SkinController.h"
#include "Collada/MorphController.h"

#include <iostream>
using namespace std;

namespace Collada {

    // Ă邩`FbN
    bool Test::testCollection( Collection* collection )
    {
        // visualScene
        const int visualSceneNumber = collection->mVisualScenes.size();
        for( int vs=0; vs<visualSceneNumber; vs++ ) {

            VisualScene* visualScene = collection->mVisualScenes[ vs ];
            if( !visualScene ) {

                cout << "err: can not find visual scene." << endl;
                return false;

            }

            if( !testVisualScene( visualScene, collection ) ) {

                cout << "err: testVisualScene()." << endl;
                return false;

            }

        }

        // controller
        const int controllerNumber = collection->mControllers.size();
        for( int ct=0; ct<controllerNumber; ct++ ) {

            Controller* controller = collection->mControllers[ ct ];
            if( !controller ) {

                cout << "err: can not find controller." << endl;
                return false;				

            }

            if( controller->mType == Controller::Type_SKIN ) {

                SkinController* skinController = dynamic_cast<SkinController*>(controller);
                if( !testSkinController( skinController, collection ) ) {

                    cout << "err: testSkinController()." << endl;
                    return false;				

                }

            }
            else if( controller->mType == Controller::Type_MORPH ) {

                MorphController* morphController = dynamic_cast<MorphController*>(controller);
                if( !testMorphController( morphController, collection ) ) {

                    cout << "err: testMorphController()." << endl;
                    return false;				

                }

            }

        }

        return true;
    }

    // Ă邩`FbN(MorphController)
    bool Test::testMorphController( MorphController* controller, Collection* collection )
    {
        VisualScene* visualScene = collection->mVisualScenes[ 0 ];

        Geometry* sourceGeometry = collection->getGeometry( controller->mSource.c_str() );
        const int sourcePolyListNumber = sourceGeometry->mPolyLists.size();
        const int sourceTrianglesNumber = sourceGeometry->mTriangles.size();

        // ^[Qbg̃eXg
        StringVector* targetVector = controller->mTarget;

        const int targetNumber = targetVector->getCount();
        for( int j=0; j<targetNumber; j++ ) {

            string targetId;
            targetId = targetVector->mElement[ j ];

            // target
            Geometry* targetGeometry = collection->getGeometry( targetId );
            if( !targetGeometry ) {

                cout << "err: can not find target geometry = " << targetId << endl;
                return false;

            }

            const int targetPolyListNumber = targetGeometry->mPolyLists.size();
            const int targetTrianglesNumber = targetGeometry->mTriangles.size();

            if( sourcePolyListNumber != targetPolyListNumber ) {

                cout << "err: polyList number is not match." << endl;
                return false;

            }
            if( sourceTrianglesNumber != targetTrianglesNumber ) {

                cout << "err: triangles number is not match. triangles = " << sourceGeometry->mId << " target = " << targetGeometry->mId << endl;
                return false;

            }

        }

        // EFCg̃eXg
        FloatVector* weightVector = controller->mWeight;
        if( !weightVector ) {

            cout << "err: can not find weight." << endl;
            return false;

        }

        return true;
    }

    // Ă邩`FbN(SkinController)
    bool Test::testSkinController( SkinController* controller, Collection* collection )
    {
        VisualScene* visualScene = collection->mVisualScenes[ 0 ];

        // ^[Qbg̃eXg
        StringVector* targetVector = controller->mTarget;		

        const int targetNumber = targetVector->getCount();
        for( int j=0; j<targetNumber; j++ ) {

            string jointId = targetVector->mElement[ j ];

            // joint
            // visualScene::getNode()ł̓[g̃m[hłȂ̂Œ
            Node* node = Util::findNode( visualScene, jointId.c_str() );
            if( !node ) {

                cout << "err: can not find node = " << jointId << endl;
                return false;

            }

        }

        // EFCg̃eXg
        FloatVector* weightVector = controller->mWeight;
        if( !weightVector ) {

            cout << "err: can not find weight." << endl;
            return false;

        }

        VertexWeights* vertexWeights = controller->mVertexWeights;
        if( !vertexWeights ) {

            cout << "err: can not find vertex weights." << endl;
            return false;

        }

        // vcount̃Xg擾
        vector< int > vcountArray = vertexWeights->mVCounts;

        IntVector* jointIndexVector = vertexWeights->mIndices[ 0 ];
        IntVector* weightIndexVector = vertexWeights->mIndices[ 1 ];

        if( !jointIndexVector || !weightIndexVector ) {

            cout << "err: can not find jointIndexVector or weightIndexVector." << endl;
            return false;

        }

        int totalVCount = 0;
        const int vcountNumber = vcountArray.size();
        for( int v=0; v<vcountNumber; v++ ) {

            totalVCount += vcountArray[ v ];

        }

        if( jointIndexVector->getCount() != totalVCount ) {

            cout << "err: invalid joint index." << endl;
            return false;		

        }

        if( weightIndexVector->getCount() != totalVCount ) {

            cout << "err: invalid weight index." << endl;
            return false;		

        }

        int count = 0;
        for( int v=0; v<vcountNumber; v++ ) {

            const int vcount = vcountArray[ v ];
            float totalWeight = 0.0f;

            for( int vc=0; vc<vcount; vc++ ) {

                const int jointIndex = jointIndexVector->mElement[ count ];

                if( targetNumber <= jointIndex ) {

                    cout << "err: invalid joint index." << endl;
                    return false;

                }

                const int weightIndex = weightIndexVector->mElement[ count ];

                if( weightVector->getCount() <= weightIndex ) {

                    cout << "err: invalid weight index." << endl;
                    return false;				

                }

                totalWeight += weightVector->mElement[ weightIndex ];

                count++;

            }

            // 10%͖Ƃ܂
            if( totalWeight < 0.1f ) {

                cout << "err: total weight too small = " << totalWeight << endl;
                return false;

            }

            // WIg
            string geometryId;
            Util::sourceToId( controller->mSource.c_str(), &geometryId );
            Geometry* geometry = collection->getGeometry( geometryId );
            if( !geometry ) {

                cout << "err: can not find geometry. geometry = " << geometryId << endl;
                return false;							

            }

            const int polylistNumber = geometry->mPolyLists.size();
            for( int pl=0; pl<polylistNumber; pl++ ) {

                PolyList* polylist = geometry->mPolyLists[ pl ];

                const int inputIndex = 0; // pos͈ԍŏ
                const Input& input = polylist->mInputs[ inputIndex ]; 
                Collada::IntVector* iv = polylist->mIndices[ input.mOffset ];

                const int indexNumber = iv->getCount();

                for( int i=0; i<indexNumber; i++ ) {

                    const int posIndex = iv->mElement[ i ];

                    if( weightVector->getCount() <= posIndex ) {

                        cout << "err: weight index over." << endl;
                        return false;

                    }


                }

            }

        }
        return true;
    }

    // Ă邩`FbN(VisualScene)
    bool Test::testVisualScene( VisualScene* visualScene, Collection* collection )
    {
        const int nodeNumber = visualScene->mNodes.size();
        for( int i=0; i<nodeNumber; i++ ) {

            Node* node = visualScene->mNodes[ i ];

            if( !node ) {

                cout << "err: can not find node." << endl;
                return false;

            }

            if( !testNode( node, collection ) ) {

                cout << "err: testNode()." << endl;
                return false;

            }

        }

        // vf𒼐ڃ`FbNĂ݂
        for( unsigned g=0; g<collection->mGeometries.size(); g++ ) {

            Geometry* geometry = collection->mGeometries[ g ];

            for( unsigned t=0; t<geometry->mTriangles.size(); t++ ) {

                Triangles* triangles = geometry->mTriangles[ t ];
                if( !collection->getMaterial( triangles->mMaterial.c_str() ) ) {

                    cout << "err: can not find material used triangles. geometry = " << geometry->mId << ", material = " << triangles->mMaterial << endl;
                    return false;

                }

            }

        }

        return true;
    }

    // Ă邩`FbN(Node)    
    bool Test::testNode( Node* node, Collection* collection ) {

        if( node->mInstanceType == Node::InstanceType_GEOMETRY ) {

            InstanceGeometry* instanceGeometry = node->mInstanceGeometry;
            if( !testInstanceGeometry( instanceGeometry, collection ) ) {

                cout << "err: testInstanceGeometry()." << endl;
                return false;

            }

        }

        for( unsigned i=0; i<node->mChildren.size(); i++ ) {

            Node* childNode = node->mChildren[ i ];
            if( !testNode( childNode, collection ) ) {

                cout << "err: testNode(). child err." << endl;
                return false;

            }

        }

        return true;
    }

    // Ă邩`FbN(InstanceGeometry)    
    bool Test::testInstanceGeometry( InstanceGeometry* instanceGeometry,
        Collection* collection )
    {
        // WIg擾
        string geometryName;
        Util::sourceToId( instanceGeometry->mURL, &geometryName );
        Geometry* geometry = collection->getGeometry( geometryName );
        if( !geometry ) {

            cout << "err: can not find geometry = " << geometryName << endl;
            return false;

        }

        const int instanceMaterialNumber = instanceGeometry->mInstanceMaterials.size();

        if( instanceMaterialNumber == 0 ) {

            cout << "err: instance material number is zero." << endl;
            cout << "err: instanceGeometry url = " << instanceGeometry->mURL << endl;
            return false;

        }

        for( int i=0; i<instanceMaterialNumber; i++ ) {

            InstanceMaterial* instanceMaterial = &instanceGeometry->mInstanceMaterials[ i ];

            string target;
            Util::sourceToId( instanceMaterial->mTarget, &target );

            // }eA݂͑H
            if( !collection->getMaterial( target ) ) {

                cout << "err: testGeometry(). material doesn't exist = " << target << endl;
                return false;

            }

            if( !testGeometry( geometry, target, collection ) ) {

                cout << "err: testGeometry(). geometry = " << geometry->mId << endl;
                return false;

            }

            Material* material = collection->getMaterial( target );
            if( !material ) {

                cout << "err: can not find material." << endl;
                return false;

            }

            if( !testMaterial( material, geometry, collection ) ) {

                cout << "err: testMaterial()." << endl;
                return false;

            }


            string instanceEffect;
            Util::sourceToId( material->mInstanceEffect, &instanceEffect );

            Effect* effect = collection->getEffect( instanceEffect );
            if( !effect ) {

                cout << "err: can not find effect." << endl;
                return false;

            }

        }

        return true;
    }

    // Ă邩`FbN(Geometry)        
    bool Test::testGeometry( Geometry* geometry, const string& materialName, Collection* collection )
    {
        string materialNameRS( materialName );

        // vf̖OdĂȂ`FbN
        int sourceNumber = geometry->getSourceNumber();
        for( int i=0; i<sourceNumber-1; i++ ) {

            const string id = geometry->getSource( i )->mId;

            for( int f=i+1; f<sourceNumber; f++ ) {

                if( id == geometry->getSource( f )->mId ) {

                    cout << "err: there are same name sources." << endl;
                    return false;

                }

            }

        }

        // PolyList
        const int polylistNumber = geometry->mPolyLists.size();
        if( polylistNumber ) {

            bool find = false;
            for( int i=0; i<polylistNumber; i++ ) {

                PolyList* polylist = geometry->mPolyLists[ i ];
                if( materialNameRS == polylist->mMaterial ) {

                    find = true;

                }

                if( !testPolyList( polylist, geometry, collection ) ) {

                    cout << "err: testPolyList()." << endl;
                    return false;

                }

            }
            if( !find ) {

                cout << "err: can not find polylist. geometry = " << geometry->mId << ", materialName = " << materialNameRS << endl;
                return false;

            }

        }

        // Triangles
        const int trianglesNumber = geometry->mTriangles.size();
        if( trianglesNumber > 0 ) {

            bool find = false;
            for( int i=0; i<trianglesNumber; i++ ) {

                Triangles* triangles = geometry->mTriangles[ i ];
                if( materialNameRS == triangles->mMaterial ) {

                    find = true;

                }

                if( !testTriangles( triangles, geometry, collection ) ) {

                    cout << "err: testTriangles()." << endl;
                    return false;

                }

            }
            if( !find ) {

                cout << "err: can not find triangles. material = " << materialNameRS << endl;
                return false;

            }

        }

        return true;
    }

    // Ă邩`FbN(PolyList)
    bool Test::testPolyList( PolyList* polylist, Geometry* geometry, Collection* collection )
    {
        // }eA݂͑H
        if( !collection->getMaterial( polylist->mMaterial ) ) {

            cout << "err: can not find material = " << polylist->mMaterial << endl;
            return false;

        }

        const int inputNumber = polylist->mInputs.size();
        for( int i=0; i<inputNumber; i++ ) {

            const Input& input = polylist->mInputs[ i ];

            if( !testInput( input, geometry ) ) {

                cout << "err: testInput()." << endl;
                return false;

            }

            if( !testGeometrySource( geometry, polylist->mIndices[ input.mOffset ], input ) ) {

                cout << "err: testInput()." << endl;
                return false;						

            }

        }

        return true;
    }

    // Ă邩`FbN(Triagles)
    bool Test::testTriangles( Triangles* triangles, Geometry* geometry, Collection* collection )
    {
        // }eA݂͑H
        if( !collection->getMaterial( triangles->mMaterial ) ) {

            cout << "err: can not find material = " << triangles->mMaterial << endl;
            return false;

        }

        const int inputNumber = triangles->mIndices.size();
        for( int i=0; i<inputNumber; i++ ) {

            const Input& input = triangles->mInputs[ i ];

            if( !testInput( input, geometry ) ) {

                cout << "err: testInput()." << endl;
                return false;

            }

            if( !testGeometrySource( geometry, triangles->mIndices[ input.mOffset ], input ) ) {

                cout << "err: testInput()." << endl;
                return false;						

            }

            // countindex̐̃`FbN
            IntVector* indexVector = triangles->mIndices[ input.mOffset ];
            if( indexVector->getCount() != (triangles->mCount*3) ) {

                cout << "err: testInput(). count != index." << endl;
                return false;						

            }


        }

        return true;
    }

    // Ă邩`FbN(Index)    		
    bool Test::testIndex( StructVector* v, IntVector* indexVector )
    {
        const int num = v->getCount() / v->getElementNumber();

        for( int i=0; i<indexVector->getCount(); i++ ) {

            const int index = indexVector->mElement[ i ];

            if( index >= num ) {

                cout << "err: testIndex(). index = " << index << ", num = " << num << endl;				
                return false;

            }

        }

        return true;
    }

    // Ă邩`FbN(Geometrysource)    	
    bool Test::testGeometrySource( Geometry* geometry,
        IntVector* indexVector,
        const Input& input )
    {
        string source;
        Util::sourceToId( input.mSource, &source );

        switch( input.mSemantic ) {

        case Semantic_BINORMAL:
            if( !testIndex( geometry->getGeoBinormal( source ), indexVector ) ) {

                cout << "err: testGeometrySource()." << endl;        
                return false;

            }
            break;

        case Semantic_COLOR:
            if( !testIndex( geometry->getColor( source ), indexVector ) ) {

                cout << "err: testGeometrySource()." << endl;
                return false;

            }
            break;

        case Semantic_NORMAL:
            if( !testIndex( geometry->getNormal( source ), indexVector ) ) {

                cout << "err: testGeometrySource()." << endl;
                return false;

            }
            break;

        case Semantic_TANGENT:
            if( !testIndex( geometry->getGeoTangent( source ), indexVector ) ) {

                cout << "err: testGeometrySource()." << endl;        
                return false;

            }
            break;

        case Semantic_TEXBINORMAL:
            if( !testIndex( geometry->getBinormal( source ), indexVector ) ) {

                cout << "err: testGeometrySource()." << endl;        
                return false;

            }
            break;

        case Semantic_TEXCOORD:
            if( !testIndex( geometry->getUV( source ), indexVector ) ) {

                cout << "err: testGeometrySource()." << endl;        
                return false;

            }
            break;

        case Semantic_TEXTANGENT:
            if( !testIndex( geometry->getTangent( source ), indexVector ) ) {

                cout << "err: testGeometrySource()." << endl;        
                return false;

            }
            break;

        case Semantic_VERTEX:
            if( !testIndex( geometry->getPositionByVertices( source ), indexVector ) ) {

                cout << "err: testGeometrySource()." << endl;
                return false;

            }
            break;
        }

        return true;
    }

    // Ă邩`FbN(Input)    
    bool Test::testInput( const Input& input, Geometry* geometry )
    {
        string source;
        Util::sourceToId( input.mSource, &source );

        StructVector* sv = NULL;

        switch( input.mSemantic ) {

        case Semantic_BINORMAL:			
            sv = geometry->getGeoBinormal( source );
            break;

        case Semantic_COLOR:
            sv = geometry->getColor( source );
            break;

        case Semantic_NORMAL:
            sv = geometry->getNormal( source );
            break;

        case Semantic_TANGENT:
            sv = geometry->getGeoTangent( source );
            break;

        case Semantic_TEXBINORMAL:
            sv = geometry->getBinormal( source );
            break;

        case Semantic_TEXCOORD:
            sv = geometry->getUV( source );
            break;

        case Semantic_TEXTANGENT:
            sv = geometry->getTangent( source );
            break;

        case Semantic_VERTEX:
            sv = geometry->getPositionByVertices( source );
            break;

        default:
            cout << "err: unknown semantic." << endl;
            return false;

        }

        if( sv == NULL ) {

            cout << "err: can not find data. source = " << source << endl;
            return false;

        }

        // Vertex̂ƂPositionoRŃ`FbN
        if( input.mSemantic == Semantic_VERTEX ) {

            Vertices* vertices = geometry->getVertices( source );
            if( source != vertices->mId ) {

                cout << "err: can not find source." << endl;
                return false;

            }

            string verticesSource;
            Util::sourceToId( vertices->mInput->mSource, &verticesSource );
            if( verticesSource != sv->mId ) {

                cout << "err: can not find source.\n" << endl;    
                return false;

            }
        }
        else {

            if( source != sv->mId ) {

                cout << "err: can not find source : " << source << endl;
                return false;

            }

        }
        return true;

    }

    // Ă邩`FbN(Material)        
    bool Test::testMaterial( Material* material, Geometry* geometry, Collection* collection )
    {
        string instanceEffect;
        Util::sourceToId( material->mInstanceEffect, &instanceEffect );

        Effect* effect = collection->getEffect( instanceEffect );
        if( !effect ) {

            cout << "err: can not find effect." << endl;
            return false;

        }

        if( !testEffect( effect, material, geometry, collection ) ) {

            cout << "err: testEffect()." << endl;
            return false;

        }

        return true;
    }

    // Ă邩`FbN(Effect)            
    bool Test::testEffect( Effect* effect, Material* material, Geometry* geometry, Collection* collection )
    {
        Profile* profile = effect->mProfile;

        if( !profile ) {

            cout << "err: can not find profile." << endl;
            return false;

        }

        Technique* technique = profile->mTechnique;
        if( !technique ) {

            cout << "err: can not find technique." << endl;
            return false;

        }

        if( !testTechnique( technique, material, profile, geometry, collection ) ) {

            cout << "err: testTechnique()." << endl;
            return false;

        }

        return true;

    }

    // Ă邩`FbN(Technique)                
    bool Test::testTechnique( Technique* technique,
        Material* material, 
        Profile* profile,
        Geometry* geometry,
        Collection* collection )
    {
        const int num = technique->mColorOrTextureOrFloats.size();
        for( int i=0; i<num; i++ ) {

            ColorOrTextureOrFloat* ctf = technique->mColorOrTextureOrFloats[ i ];
            if( !ctf ) {

                cout << "err: can not find color or texture or float." << endl;
                return false;

            }

            if( ctf->mType == ColorOrTextureOrFloat::Type_TEXTURE ) {

                Texture* texture = ctf->mTexture;
                if( !texture ) {

                    cout << "err: can not find texture." << endl;
                    return false;

                }
                if( !testTexture( texture, material, profile, geometry, collection ) ) {

                    cout << "err: testTexture()." << endl;
                    return false;

                }

            }

        }

        return true;
    }

    // Ă邩`FbN(Texture)                    
    bool Test::testTexture( Texture* texture,
        Material* material, 
        Profile* profile,
        Geometry* geometry,
        Collection* collection )
    {
        // texcoord̃`FbN
        {
            // }bvȂǂtextureCoord=""ɂȂĂ̂ŒӁB
            Extra* extra = material->mExtra;
            if( !extra ) {

                cout << "err: invalid material." << endl;
                return false;

            }
            vector< ExtraAttribute* > extraAttributeArray;
            extra->getExtraAttribute( "TexCoordInfo", &extraAttributeArray );

            bool findTexCoord = false;

            string textureName( texture->mTexture );
            string geometryId( geometry->mId );


            // ̒iKł̓WIgɃ}eAPolyList\B
            for( unsigned i=0; i<extraAttributeArray.size(); i++ ) {

                const Attribute* attrTexture = extraAttributeArray[ i ]->getAttribute( "texture" );
                const Attribute* attrGeometry = extraAttributeArray[ i ]->getAttribute( "geometry" );
                const Attribute* attrTexCoord = extraAttributeArray[ i ]->getAttribute( "texcoord" );

                if( attrTexture->mName.size() == 0 || attrGeometry->mName.size() == 0 ) {

                    cout << "err: invalid attribute." << endl;
                    return false;

                }

                if( textureName != attrTexture->mValue ||
                    geometryId != attrGeometry->mValue ) {

                        continue;

                }

                // texcoord͋̏ꍇi}bv̂Ƃj
                if( attrTexCoord->mValue.size() == 0 ) {

                    // ̂Ƃ̓WIgUV݂Ȃ̂ŁAG[ł͂Ȃ
                    findTexCoord = true;
                    break;

                }

                // S"TexCoordInfo"`FbNȂƂȂ
                Collada::Float2Vector* uvVector = geometry->getUV( attrTexCoord->mValue );
                if( uvVector ) {

                    findTexCoord = true;
                    break;

                }
                else {

                    cout << "err: can not find uv = " << attrTexCoord->mValue << " geometry = " << geometry->mId << endl;
                    return false;

                }

            }

            if( !findTexCoord ) {

                cout << "err: can not find texcoord.\n";
                cout << "material = " << material->mId << ", ";
                cout << "geometry = " << geometry->mId << ", ";
                cout << "texture = " << texture->mTexture << ".\n";
                return false;

            }

        }

        // texturesampler
        const Sampler2D sampler = profile->getSampler2D( texture->mTexture );
        if( sampler.mSId == "" ) {

            cout << "err: invalid sampler.sid." << endl;
            return false;

        }

        // samplersurface
        const Surface& surface = profile->getSurface( sampler.mSource );
        if( surface.mSId == "" ) {

            cout << "err: invalid surface.sid." << endl;
            return false;

        }

        // surfaceimage
        Image* image = collection->getImage( surface.mInitFrom );
        if( !image ) {

            cout << "err: can not find image." << endl;
            return false;

        }
        if( image->mInitFrom == "" ) {

            cout << "err: invalid init from." << endl;
            return false;

        }

        return true;
    }

}