// Copyright(C)SEGA 
#include "CESystem.h"
#include "CEGeometry.h"
#include "CEMacro.h"
#include "CEUtil.h"
#include "CESystem.h"
#include "CEOption.h"

#include "Collada/Geometry.h"
#include "Collada/Accessor.h"
#include "Collada/PolyList.h"
#include "Collada/StructVector.h"
#include "Collada/Macro.h"
#include "Collada/Collection.h"
#include "Collada/Vertices.h"

#include <xsi_application.h>
#include <xsi_x3dobject.h>
#include <xsi_primitive.h>
#include <xsi_polygonmesh.h> 
#include <xsi_polygonnode.h>
#include <xsi_envelope.h>
#include <xsi_transformation.h>
#include <xsi_statickinematicstate.h>
#include <xsi_value.h>
#include <xsi_sceneitem.h>
#include <xsi_material.h>

#include <list>
#include <vector>
#include <algorithm>
#include <string>
#include <set>

using namespace XSI;
using namespace std;

// WIg擾
bool CEGeometry::get( SceneItem& sceneItem )
{
    Application app;

    // X3DObject擾
    CRef ref( sceneItem );
    X3DObject x3DObject( ref );
    VALID_FALSE( x3DObject );

    // Mesh擾
    PolygonMesh polygonMesh( x3DObject.GetActivePrimitive().GetGeometry() );
    VALID_FALSE( polygonMesh );

    app.LogMessage( L"msg: geom > " + x3DObject.GetFullName() );

    // Gx[v̉
    CRefArray envelopes;
    muteEnvelope( x3DObject, true );

    // Ŝ̒_擾
    // VFCv̉e󂯂ȂAIWĩWIg擾
    addGeometry( x3DObject, siConstructionModeModeling, false, "" );

    // Gx[v̕A
    muteEnvelope( x3DObject, false );

    return	true;
}

// ʒu擾
void CEGeometry::getPositions( CGeometryAccessor& ga, Collada::Geometry* colladaGeometry )
{
    // ʒu̔z擾
    // position0( x, y, z ), position1( x, y, z ) ... positionN( x, y, z )
    CDoubleArray positions;
    ga.GetVertexPositions( positions );

    // vf̔z
    Collada::Float3Vector* fv = new Collada::Float3Vector();
    colladaGeometry->mPositions.push_back( fv );

    // O
    fv->mId = colladaGeometry->mId + "-positions";
    fv->mName = colladaGeometry->mName + "-positions";
    fv->mArrayName = fv->mId + "-array";

    // ʒu
    const int positionNumber = positions.GetCount();
    for( int i=0; i<positionNumber; i+=3 ) {

        fv->mElement.push_back( Collada::Float3( static_cast< float >( positions[ i ] ),
            static_cast< float >( positions[ i+1 ] ),
            static_cast< float >( positions[ i+2 ] ) ) );

    }

    // Accessor
    {
        Collada::Accessor* cfAccessor = new Collada::Accessor();
        fv->mAccessor = cfAccessor;

        cfAccessor->mSource = "#" + fv->mArrayName;
        cfAccessor->mCount = fv->mElement.size(); // (x,y,z)PZbgƂ̗vf̃TCYB
        cfAccessor->mStride = 3;
        cfAccessor->mParams.push_back( Collada::Param( "X", "float" ) );
        cfAccessor->mParams.push_back( Collada::Param( "Y", "float" ) );
        cfAccessor->mParams.push_back( Collada::Param( "Z", "float" ) );
    }
}

// @擾
void CEGeometry::getNormals( CGeometryAccessor& ga, Collada::Geometry* colladaGeometry )
{
    // @̔z擾
    // normal0( x, y, z ), normal1( x, y, z ) ... normalN( x, y, z )
    CFloatArray normals;
    ga.GetNodeNormals( normals );

    // vf̔z
    Collada::Float3Vector* fv = new Collada::Float3Vector();
    colladaGeometry->mNormals.push_back( fv );

    // O
    fv->mId = colladaGeometry->mId + "-normals";
    fv->mName = colladaGeometry->mName + "-normals";
    fv->mArrayName = fv->mId + "-array";

    // @
    const int normalNumber = normals.GetCount();
    for( int i=0; i<normalNumber; i+=3 ) {

        fv->mElement.push_back( Collada::Float3( static_cast< float >( normals[ i ] ),
            static_cast< float >( normals[ i+1 ] ),
            static_cast< float >( normals[ i+2 ] ) ) );

    }

    // Accessor
    {
        Collada::Accessor* cfAccessor = new Collada::Accessor();
        fv->mAccessor = cfAccessor;

        cfAccessor->mSource = "#" + fv->mArrayName;
        cfAccessor->mCount = fv->mElement.size();
        cfAccessor->mStride = 3;
        cfAccessor->mParams.push_back( Collada::Param( "X", "float" ) );
        cfAccessor->mParams.push_back( Collada::Param( "Y", "float" ) );
        cfAccessor->mParams.push_back( Collada::Param( "Z", "float" ) );
    }
}

// _J[擾
void CEGeometry::getVertexColors( CGeometryAccessor& ga, Collada::Geometry* colladaGeometry )
{
    // _J[Zbg̔z擾
    // i_J[Zbg͕݂j
    // VertexColorSet0( vertexColor0( r, g, b, a ) ... vertexColorN( r, g, b, a ) )
    // ...
    // VertexColorSetM( vertexColor0( r, g, b, a ) ... vertexColorN( r, g, b, a ) )	
    CRefArray vertexColors = ga.GetVertexColors();

    // _J[ij
    const int vertexColorsNumber = vertexColors.GetCount();
    for( int vc=0; vc<vertexColorsNumber; vc++ ) {

        // _J[
        ClusterProperty vertexColorProperty = vertexColors[ vc ];

        // "Tangents","Binormals"ƂO̒_J[͖
        // iTangent,BinormalƂďo͂̂Łj
        if( vertexColorProperty.GetName() == L"Tangents" || vertexColorProperty.GetName() == L"Binormals" ) {

            continue;

        }

        // _J[̒l̔z
        CFloatArray values;
        vertexColorProperty.GetValues( values );		

        // vf̔z		
        Collada::ColorVector* cv = new Collada::ColorVector();
        colladaGeometry->mColors.push_back( cv );

        // Oi_J[̖̂OŌɂdlj
        const string name = vertexColorProperty.GetName().GetAsciiString();

        cv->mId = colladaGeometry->mId + "-" + name;
        cv->mName = colladaGeometry->mName + "-" + name;
        cv->mArrayName = cv->mName + "-array";

        // _J[
        const int vertexColorNumber = values.GetCount();
        for( int i=0; i<vertexColorNumber; i+=4 ) {

            // ͈̓`FbNiO`P͈̔͊O̒l邱Ƃj
            float r = static_cast<float>( values[ i ] );
            float g = static_cast<float>( values[ i+1 ] );
            float b = static_cast<float>( values[ i+2 ] );
            float a = static_cast<float>( values[ i+3 ] );        
            CEUtil::setRange0To1( &r );
            CEUtil::setRange0To1( &g );
            CEUtil::setRange0To1( &b );
            CEUtil::setRange0To1( &a );		

            cv->mElement.push_back( Collada::Color( r, g, b, a ) );

        }

        // Accessor
        {
            Collada::Accessor* cfAccessor = new Collada::Accessor();
            cv->mAccessor = cfAccessor;

            cfAccessor->mSource = "#" + cv->mArrayName;
            cfAccessor->mCount = cv->mElement.size();
            cfAccessor->mStride = 4;
            cfAccessor->mParams.push_back( Collada::Param( "R", "float" ) );
            cfAccessor->mParams.push_back( Collada::Param( "G", "float" ) );
            cfAccessor->mParams.push_back( Collada::Param( "B", "float" ) );
            cfAccessor->mParams.push_back( Collada::Param( "A", "float" ) );
        }
    }
}

// ^WFg擾
void CEGeometry::getTangents( CGeometryAccessor& ga, Collada::Geometry* colladaGeometry )
{
    // TODO:
    // ۂɂNormalMap.TangentsQƂȂƂȂ̂Ał͎蔲Ƃ
    // "Tangents"ƂÕvpeB擾ĂB
    ClusterProperty tangent = ga.GetVertexColors().GetItem( L"Tangents" );

    // ݂ȂĂG[ł͂Ȃ
    VALID_RETURN( tangent );

    CClusterPropertyElementArray arrayElement = tangent.GetElements();
    CDoubleArray array = arrayElement.GetArray();

    int	count = 0;    
    const int num = arrayElement.GetCount();

    // floatshort
    bool floatDataType = false;
    {
        CRef ref;
        ref.Set( tangent.GetFullName() + L".vertexcolorchangedatatype.desireddatatype" );
        if( ref.IsValid() ) {

            Parameter paramDesireDataType( ref );
            if( paramDesireDataType.IsValid() ) { 

                const int DESIRE_DATA_TYPE_SHORT = 0; 
                const int DESIRE_DATA_TYPE_FLOAT = 1;  

                const int value = paramDesireDataType.GetValue();

                if( value == DESIRE_DATA_TYPE_SHORT ) {

                    floatDataType = false;

                }
                else if( value == DESIRE_DATA_TYPE_FLOAT ) {

                    floatDataType = true;

                }

            }
        } 
    }

    // vf̔z
    Collada::Float3Vector* fv = new Collada::Float3Vector();
    colladaGeometry->mTangents.push_back( fv );

    // O
    fv->mId = colladaGeometry->mId + "-tangents";
    fv->mName = colladaGeometry->mName + "-tangents";
    fv->mArrayName = fv->mName + "-array";

    if( floatDataType ) {

        // Tangents͂SvfȂ̂ŒӁI
        for( int c=0; c<num; c++ ) {

            Collada::Float3 tan;
            tan.x = static_cast< float >( array[ count++ ] );
            tan.y = static_cast< float >( array[ count++ ] ); 
            tan.z = static_cast< float >( array[ count++ ] );
            count++; // ΂
            fv->mElement.push_back( tan );

        }

    }
    else {

        // Tangents͂SvfȂ̂ŒӁI	
        for( int c=0; c<num; c++ ) {

            // short̂ƂfloatLXgŒl炵
            float x = static_cast< float >( array[ count++ ] );
            float y = static_cast< float >( array[ count++ ] );
            float z = static_cast< float >( array[ count++ ] );
            count++; // ΂			

            // short̏ꍇ0...1-1...1ɕϊ
            x	= x*2.0f - 1.0f; 
            y	= y*2.0f - 1.0f;
            z	= z*2.0f - 1.0f;

            Collada::Float3 tan( x, y, z );
            fv->mElement.push_back( tan );

        }

    }

    // Accessor(tangent)
    {
        Collada::Accessor* cfAccessor = new Collada::Accessor();
        fv->mAccessor = cfAccessor;

        cfAccessor->mSource = "#" + fv->mArrayName;
        cfAccessor->mCount = fv->mElement.size();
        cfAccessor->mStride = 3;
        cfAccessor->mParams.push_back( Collada::Param( "X", "float" ) );
        cfAccessor->mParams.push_back( Collada::Param( "Y", "float" ) );
        cfAccessor->mParams.push_back( Collada::Param( "Z", "float" ) );
    }

}

// ]@擾
void CEGeometry::getBinormals( CGeometryAccessor& ga, Collada::Geometry* colladaGeometry )
{
    // TODO:
    // "Binormals"ƂOŌŒ肵Ă
    ClusterProperty binormal = ga.GetVertexColors().GetItem( L"Binormals" );

    // ݂ȂĂG[ł͂Ȃ
    VALID_RETURN( binormal );

    CClusterPropertyElementArray arrayElement = binormal.GetElements();
    CDoubleArray array = arrayElement.GetArray();

    int	count = 0;    
    const int num = arrayElement.GetCount();

    // floatshort  
    bool floatDataType = false;
    {
        CRef ref;
        ref.Set( binormal.GetFullName() + L".vertexcolorchangedatatype.desireddatatype" );
        if( ref.IsValid() ) {

            Parameter paramDesireDataType( ref );
            if( paramDesireDataType.IsValid() ) { 

                const int DESIRE_DATA_TYPE_SHORT = 0; 
                const int DESIRE_DATA_TYPE_FLOAT = 1;  

                const int value = paramDesireDataType.GetValue();

                if( value == DESIRE_DATA_TYPE_SHORT ) {

                    floatDataType = false;

                }
                else if( value == DESIRE_DATA_TYPE_FLOAT ) {

                    floatDataType = true;

                }

            }
        } 
    }

    // vf̔z
    Collada::Float3Vector* fv = new Collada::Float3Vector();	
    colladaGeometry->mBinormals.push_back( fv );	

    // O
    fv->mId = colladaGeometry->mId + "-binormals";
    fv->mName = colladaGeometry->mName + "-binormals";
    fv->mArrayName = fv->mName + "-array";

    if( floatDataType ) {

        // Binormal͂SvfȂ̂Œ
        for( int c=0; c<num; c++ ) {

            Collada::Float3 tan;
            tan.x = static_cast< float >( array[ count++ ] );
            tan.y = static_cast< float >( array[ count++ ] ); 
            tan.z = static_cast< float >( array[ count++ ] );
            count++;
            fv->mElement.push_back( tan );

        }

    }
    else {

        // Binormal͂SvfȂ̂Œ	
        for( int c=0; c<num; c++ ) {

            float x = static_cast< float >( array[ count++ ] );
            float y = static_cast< float >( array[ count++ ] );
            float z = static_cast< float >( array[ count++ ] );
            count++;
            x	= x*2.0f - 1.0f;
            y	= y*2.0f - 1.0f;
            z	= z*2.0f - 1.0f;

            Collada::Float3 tan( x, y, z );
            fv->mElement.push_back( tan );

        }

    }

    // Accessor(binormal)
    {
        Collada::Accessor* cfAccessor = new Collada::Accessor();
        fv->mAccessor = cfAccessor;

        cfAccessor->mSource = "#" + fv->mArrayName;
        cfAccessor->mCount = fv->mElement.size();
        cfAccessor->mStride = 3;
        cfAccessor->mParams.push_back( Collada::Param( "X", "float" ) ); 
        cfAccessor->mParams.push_back( Collada::Param( "Y", "float" ) );
        cfAccessor->mParams.push_back( Collada::Param( "Z", "float" ) );
    }
}

// UV擾ij
void CEGeometry::getUVs( CGeometryAccessor& ga, Collada::Geometry* colladaGeometry )
{
    // UVZbg̔z擾
    // iUVZbg͕݂j
    // UVSet0( uv0( u, v, w ) ... uvN( u, v, w ) )
    // ...
    // UVSetM( uv0( u, v, w ) ... uvN( u, v, w ) )	
    CRefArray uvs = ga.GetUVs();

    const int uvsNumber = uvs.GetCount();
    for( int uvsIndex = 0; uvsIndex<uvsNumber; uvsIndex++ ) {

        // UV
        ClusterProperty uvProperty = uvs[ uvsIndex ];

        // UV̒l̔z		
        CFloatArray values;
        uvProperty.GetValues( values );

        // vf̔z		
        Collada::Float2Vector* fv = new Collada::Float2Vector();
        colladaGeometry->mUVs.push_back( fv );

        // OiUV̖̂OŌɂdlj
        const string name = uvProperty.GetName().GetAsciiString();

        fv->mId = colladaGeometry->mId + "-" + name;
        fv->mName = colladaGeometry->mName + "-" + name;
        fv->mArrayName = fv->mName + "-array";

        // UV
        // UV(u,v,w)̂RȂ̂Œ
        const int uvNumber = values.GetCount();
        for( int i=0; i<uvNumber; i+=3 ) {

            // w͎gȂ
            fv->mElement.push_back( Collada::Float2( static_cast< float >( values[ i ] ),
                static_cast< float >( values[ i+1 ] ) ) );

        }

        // Accessor
        {
            Collada::Accessor* cfAccessor = new Collada::Accessor();
            fv->mAccessor = cfAccessor;

            cfAccessor->mSource = "#" + fv->mArrayName;
            cfAccessor->mCount = fv->mElement.size();
            cfAccessor->mStride = 2;            
            cfAccessor->mParams.push_back( Collada::Param( "S", "float" ) );
            cfAccessor->mParams.push_back( Collada::Param( "T", "float" ) );
        }
    }
}

// PolyList擾
void CEGeometry::getPolyLists( CGeometryAccessor& ga,
                              Collada::Geometry* colladaGeometry )
{
    // e|SƂ̒_擾
    // poly0( i0, i1, i2 ), poly1( i0, i1, i2, i3 ) ... polyN( i0, i1, i2 )
    // ̏ꍇ͈ȉ̂悤ȒlĂ
    // 3, 4, ... 3
    CLongArray polygonVertexNumbers;
    ga.GetPolygonVerticesCount( polygonVertexNumbers );

    // |S̒_CfbNXz擾
    // poly0( i0, i1, i2 ), poly1( i0, i1, i2, i3 ) ... polyN( i0, i1, i2 )
    // |S̏ꍇA_CfbNX̌̓|SƂɈႤ̂Œ
    // _CfbNX͈ʒũCfbNXƂĎgp	
    CLongArray vertexIndices;
    ga.GetVertexIndices( vertexIndices );

    // |S̃m[hCfbNXz擾
    // poly0( i0, i1, i2 ), poly1( i0, i1, i2, i3 ) ... polyN( i0, i1, i2 )	
    // |S̏ꍇAm[hCfbNX̌̓|SƂɈႤ̂Œ
    // m[hCfbNX͈ʒuȊÕCfbNXƂĎgp
    CLongArray nodeIndices;
    ga.GetNodeIndices ( nodeIndices );

    // PolygonMeshgpĂ}eAij擾
    // CGeometryAccessor::GetMaterials()̌ʂ͏dĂ邱Ƃ̂Œ
    CRefArray materials = ga.GetMaterials();

    // }eAd`FbNp̃Zbg
    set< string > materialNames;

    // }eAƂPolyList or Triangles\zĒǉ
    const int materialNumber = materials.GetCount(); 
    for( int materialIndex=0; materialIndex<materialNumber; materialIndex++ ) {

        Material material = materials[ materialIndex ];

        // d`FbN
        const string materialName = material.GetName().GetAsciiString();
        if( materialNames.end() != materialNames.find( materialName ) ) {

            continue;

        }
        else {

            materialNames.insert( materialName );

        }

        // ColladaPolyList𐶐
        Collada::PolyList* polylist = new Collada::PolyList();

        // }eA		
        polylist->mMaterial = material.GetName().GetAsciiString();

        // ̃}eAgpĂ|S̃CfbNX擾
        // CBitArraybool̔zȂ̂ŒӁB
        // index̂܂ܓĂ킯ł͂ȂB
        CBitArray polygonIndices;
        ga.GetPolygonMaterialIndicesByMaterial( material, polygonIndices );

        // CfbNX̃xN^
        Collada::IntVector* vertexIndicesVector = new Collada::IntVector();
        Collada::IntVector* nodeIndicesVector = new Collada::IntVector();

        // |SPʂŃ[v
        int offset = 0;		
        const int polygonNumber = polygonIndices.GetCount();		
        for( int polygonIndex=0; polygonIndex<polygonNumber; polygonIndex++ ) {

            // ̃|S̒_
            const int vertexNumber = polygonVertexNumbers[ polygonIndex ];				

            // polygonIndicesfalsê͖̂
            if( !polygonIndices[ polygonIndex ] ) {

                // _i߂			
                offset += vertexNumber;
                continue;

            }

            // |SPʂ̒_ǉ		
            polylist->mVCounts.push_back( vertexNumber );

            // _PʂŃ[v
            for( int vertexIndex=0; vertexIndex<vertexNumber; vertexIndex++ ) {

                vertexIndicesVector->mElement.push_back( vertexIndices[ offset + vertexIndex ] );
                nodeIndicesVector->mElement.push_back( nodeIndices[ offset + vertexIndex ] );

            }

            // _i߂
            offset += vertexNumber;

        }

        // polylistɃCfbNX̔zǉ
        polylist->mIndices.push_back( vertexIndicesVector );
        polylist->mIndices.push_back( nodeIndicesVector );

        // Input
        {
            // pos
            polylist->mInputs.push_back( Collada::Input( Collada::Semantic_VERTEX, // Positionł͂Ȃ̂Œ
                "#" + colladaGeometry->mId + "-vertices",
                0 ) ); // Offset0ŌŒ

            // nrm
            polylist->mInputs.push_back( Collada::Input( Collada::Semantic_NORMAL,
                "#" + colladaGeometry->mId + "-normals",
                1 ) ); // Offset1ŌŒ

            // color
            const int colorsNumber = colladaGeometry->mColors.size();
            for( int color=0; color<colorsNumber; color++ ) {

                polylist->mInputs.push_back( Collada::Input( Collada::Semantic_COLOR,
                    "#" + colladaGeometry->mColors[ color ]->mId,
                    1 ) );

            }

            // tangent
            const int tangentsNumber = colladaGeometry->mTangents.size();
            for( int tangent=0; tangent<tangentsNumber; tangent++ ) {

                polylist->mInputs.push_back( Collada::Input( Collada::Semantic_TEXTANGENT,
                    "#" + colladaGeometry->mTangents[ tangent ]->mId,
                    1 ) ); 

            }

            // binormal
            const int binormalNumber = colladaGeometry->mBinormals.size();
            for( int binormal=0; binormal<binormalNumber; binormal++ ) {

                polylist->mInputs.push_back( Collada::Input( Collada::Semantic_TEXBINORMAL,
                    "#" + colladaGeometry->mBinormals[ binormal ]->mId,
                    1 ) );

            }

            // uv
            const int uvsNumber = colladaGeometry->mUVs.size();
            for( int uv=0; uv<uvsNumber; uv++ ) {

                polylist->mInputs.push_back( Collada::Input( Collada::Semantic_TEXCOORD,
                    "#" + colladaGeometry->mUVs[ uv ]->mId,
                    1,
                    uv ) ); // UV̂setKv

            }

        }

        // |SPȏȂǉ
        // ĩNX^[Ăꍇ̂Łj
        if( polylist->mIndices[ 0 ]->mElement.size() > 0 ) {

            colladaGeometry->mPolyLists.push_back( polylist );

        }
        else {

            delete polylist;
            polylist = NULL;

        }

    }

}

// Vertices擾
void CEGeometry::getVertices( Collada::Geometry* colladaGeometry )
{
    Collada::Vertices* vertices = new Collada::Vertices();
    colladaGeometry->mVertices.push_back( vertices );

    vertices->mId = colladaGeometry->mId + "-vertices";
    vertices->mName = colladaGeometry->mName + "-vertices";

    vertices->mInput = new Collada::Input( Collada::Semantic_POSITION,
        "#" + colladaGeometry->mId + "-positions",
        0 );
}

// Gx[v̐ݒύX
void CEGeometry::muteEnvelope( X3DObject& x3DObject,
                              const bool enableMute )
{
    CRefArray envelopes = x3DObject.GetEnvelopes();
    const int num = envelopes.GetCount();

    for( int i=0; i<num; i++ ) {

        Envelope envelope = envelopes[ i ];
        if( envelope.GetName() == L"Envelope Operator" ) {

            Parameter paramMute = envelope.GetParameter( L"mute" );
            if( paramMute.IsValid() ) {

                paramMute.PutValue( enableMute );

            } 
        }

    }

}

// NX^[ɑTextureProjection擾
void CEGeometry::getTextureProjections( Cluster& cluster,
                                       CRefArray& textureProjections )
{
    // ̃NX^[Tv[ȂƂ͉Ȃ
    if( cluster.GetType() !=  L"sample" ) {

        return;

    }

    const int num = cluster.GetProperties().GetCount();
    for( int i=0;i <num; i++ ) {

        Property property = Property( cluster.GetProperties()[ i ] );
        if( property.GetType() == L"uvspace" ) {

            textureProjections.Add( property );

        }
    }
}

// TextureProjection炻ɕRTextureProjectionDef擾
void CEGeometry::getTextureProjectionDefs( const Property& textureProjection,
                                          Primitive& textureProjectionDef )
{
    CRefArray nestedObjects = textureProjection.GetNestedObjects();
    for( int i=0; i<nestedObjects.GetCount(); i++ ) {

        if( nestedObjects[ i ].GetClassIDName() == L"Primitive" ) {

            Primitive primitive = Primitive( nestedObjects[ i ] );
            VALID_CONTINUE( primitive );

            if( primitive.GetType() == L"uvprojdef" ) {

                textureProjectionDef = primitive;
                break;

            }

        }
    }
}

// UVTransform擾
void CEGeometry::getUVTransform( Geometry& geometry )
{
    CRefArray sampleClusters;
    geometry.GetClusters().Filter( CString( L"sample" ),
        CStringArray(),
        CString(),
        sampleClusters );

    for( int i=0; i<sampleClusters.GetCount(); i++ ) {

        Cluster cluster( sampleClusters[ i ] );

        CRefArray textureProjections;
        getTextureProjections( cluster, textureProjections );

        for( int i=0; i<textureProjections.GetCount(); i++ ) {

            Property property = textureProjections[ i ];
            VALID_CONTINUE( property );

            Primitive textureProjectionDef;
            getTextureProjectionDefs( property, textureProjectionDef );
            VALID_CONTINUE( textureProjectionDef );

            Parameter paramScaleU;
            Parameter paramScaleV;
            Parameter paramOffsetU;
            Parameter paramOffsetV;

            paramScaleU = textureProjectionDef.GetParameter( L"projsclu" );
            paramScaleV = textureProjectionDef.GetParameter( L"projsclv" );
            paramOffsetU = textureProjectionDef.GetParameter( L"projtrsu" );
            paramOffsetV = textureProjectionDef.GetParameter( L"projtrsv" );

        }

    }
}

// WIgǉ
bool CEGeometry::addGeometry( X3DObject x3DObject,
                             const siConstructionMode mode,
                             const bool shapeMode,
                             const string& shapeName )
{
    // ColladaGeometry𐶐Ēǉ
    Collada::Geometry* colladaGeometry = new Collada::Geometry();
    CESystem::get()->getCollection()->mGeometries.push_back( colladaGeometry );

    // O
    string geometryId;
    CEUtil::getRootPath( x3DObject, &geometryId );    
    const string geometryName = x3DObject.GetName().GetAsciiString();    

    colladaGeometry->mId = geometryId;
    colladaGeometry->mName = geometryName;

    // VFCv̂Ƃ̓VFCvŌɂ
    if( shapeMode ) {

        colladaGeometry->mId += "-" + shapeName;
        colladaGeometry->mName += "-" + shapeName;

    }

    // x3DObjectPolygonMesh擾
    PolygonMesh polygonMesh( x3DObject.GetActivePrimitive().GetGeometry() );
    CGeometryAccessor ga = polygonMesh.GetGeometryAccessor( mode );

    // Primitive擾
    Primitive primitive = x3DObject.GetActivePrimitive();
    VALID_FALSE( primitive );

    // Geometry擾
    Geometry geometry = primitive.GetGeometry( DBL_MAX, mode );
    VALID_FALSE( geometry );

    // ʒu
    getPositions( ga, colladaGeometry );

    // @
    getNormals( ga, colladaGeometry );

    // VFCv̏ꍇ͈ʒuƖ@̂
    if( !shapeMode ) {

        // UV
        getUVs( ga, colladaGeometry );

        // _J[
        getVertexColors( ga, colladaGeometry );

        // ڐ
        getTangents( ga, colladaGeometry );

        // ]@
        getBinormals( ga, colladaGeometry );		

        // UVTransform̏擾
        getUVTransform( geometry );

    }

    // CfbNX
    getPolyLists( ga, colladaGeometry );

    // Vertices
    getVertices( colladaGeometry );

    return true;
}
