// Copyright(C)SEGA 
#include "CEObject.h"
#include "CEUtil.h" 
#include "CESystem.h"
#include "CEMacro.h"
#include "CEOption.h"
#include "CEGeometry.h"

#include "Collada/Collection.h"
#include "Collada/Node.h"
#include "Collada/VisualScene.h"
#include "Collada/StructVector.h"
#include "Collada/Controller.h"
#include "Collada/Extra.h"
#include "Collada/Camera.h"
#include "Collada/Light.h"
#include "Collada/Geometry.h"
#include "Collada/PolyList.h"
#include "Collada/InstanceGeometry.h"

#include <xsi_application.h>
#include <xsi_sceneitem.h>
#include <xsi_camera.h>
#include <xsi_light.h>
#include <xsi_material.h>
#include <xsi_polygonmesh.h>

using namespace XSI;
using namespace std;

// m[h擾
bool CEObject::get()
{
    Collada::Collection* collection = CESystem::get()->getCollection();
    assert( collection );

    Application app;
    app.LogMessage( L"msg: getObject" );

    CRefArray selection = CESystem::get()->getSelection();
    VALID_TRUE( selection );

    // m[hArray
    vector< Collada::Node* > nodeArray;

    // eArray
    vector< string > parentIdArray;

    // ZNgXg擾Ă
    const int selectionCount = selection.GetCount();
    for( int i=0; i<selectionCount; i++ ) {

        // SceneItem擾
        SceneItem	sceneItem = selection.GetItem( i );
        VALID_CONTINUE( sceneItem );

        // 擾IuWFNgH
        if( !CEUtil::isGetObject( sceneItem ) ) {

            continue;

        }

        // IuWFNg̏ڍׂ擾
        Collada::Node* cfNode = new Collada::Node();
        string parentId( "" );

        if( !getObjectInfo( sceneItem, cfNode, &parentId ) ) {

            return false;

        }

        nodeArray.push_back( cfNode );
        parentIdArray.push_back( parentId );

    }

    // S̎擾IĂNodeType肷
    const int nodeArrayNumber = nodeArray.size();
    for( int i=0; i<nodeArrayNumber; i++ ) {

        Collada::Node* cfNode = nodeArray[ i ];

        // EFCg̑ΏۂȂJOINT, łȂȂNULLɂȂ
        if( isWeightTarget( cfNode->mId ) ) {

            cfNode->mNodeType = Collada::Node::NodeType_JOINT;

        }
        else {

            cfNode->mNodeType = Collada::Node::NodeType_NODE;

        }

    }

    // eqlĊi[
    Collada::VisualScene* visualScene = collection->mVisualScenes[ 0 ];
    assert( visualScene );

    addNode( nodeArray, parentIdArray, visualScene );

    return	true;
}

// Node_EFCg̑ΏۂǂԂ
bool CEObject::isWeightTarget( const string& name )
{
    Collada::Collection* cfCollection = CESystem::get()->getCollection();

    // Rg[[EFCg̑ΏۂǂׂB
    const int controllerNumber = cfCollection->mControllers.size();
    for( int i=0; i<controllerNumber; i++ ) {

        Collada::Controller* cfController = cfCollection->mControllers[ i ];
        Collada::StringVector* jointVector = cfController->mTarget;

        for( int j=0; j<jointVector->getCount(); j++ ) {

            if( name == jointVector->mElement[ j ] ) {

                return true;

            }


        }

    }

    return false;
}

// eqlĊi[
void CEObject::addNode( vector< Collada::Node* >& nodeArray,
                       vector< string >& parentIdArray,
                       Collada::VisualScene* visualScene )
{
    // ܂[g̉ɂȂ̂֘At
    const int nodeNumber = nodeArray.size();
    for( int i=0; i<nodeNumber; i++ ) {

        Collada::Node* cfNode = nodeArray[ i ];
        if( !cfNode ) {

            continue;

        }

        // eScene_RootȊÔ̂
        string parentName( parentIdArray[ i ] );
        if( parentName != "Scene_Root" ) {

            // e
            bool find = false;
            for( int p=0; p<nodeNumber; p++ ) {

                if( nodeArray[ p ] == NULL ) {

                    continue;

                }

                if( parentIdArray[ i ] == nodeArray[ p ]->mId ) {

                    // qƂĒǉ
                    nodeArray[ p ]->mChildren.push_back( cfNode );

                    find = true;
                    break;

                }

            }
            // eȂ疳
            if( !find ) {

                const string msg = "can not find parent. node = " + cfNode->mId + " " + "parent = " + parentIdArray[ i ] + "\n";

                Application app;
                app.LogMessage( msg.c_str() );

                CE_SAFE_DELETE( nodeArray[ i ] );

            }

        }

    }

    // [ĝ̂VisualSceneɒǉ
    for( int i=0; i<nodeNumber; i++ ) {

        // eScene_RootȊÔ̂ǉ
        if( parentIdArray[ i ] == "Scene_Root" ) {

            visualScene->mNodes.push_back( nodeArray[ i ] ); 

        }

    } 

}

// IuWFNg̏ڍׂ擾
bool CEObject::getObjectInfo( SceneItem& sceneItem,
                             Collada::Node* cfNode,
                             string* parentId )
{
    // ^Cv
    const CString type = sceneItem.GetType();

    // O
    string id;
    CEUtil::getRootPath( sceneItem, &id );    
    const string name = sceneItem.GetName().GetAsciiString();

    cfNode->mId = id;
    cfNode->mName = name;

    // |SbV
    if( type == L"polymsh" || type == L"poly" ) {

        // fo͂H
        if( CESystem::get()->getOption()->getExportModel() ) {

            // WIg擾
            // polymeshłWIg擾łȂ\̂ŒӁB
            // |S폜ꍇȂǁB
            if( !getGeometry( sceneItem, cfNode ) ) {

                Application app;
                app.LogMessage( L"err: getGeometry." );
                return false;

            }

        }
        cfNode->mNodeType = Collada::Node::NodeType_NODE;

    }
    // Cg
    else if( type == L"light" ) {

        // Cgo͂H
        if( CESystem::get()->getOption()->getExportLight() ) {

            if( !getLight( sceneItem, cfNode ) ) {

                Application app;
                app.LogMessage( L"err: getLight." );
                return false;				

            }

        }

    }
    // J
    else if( type == L"camera" ) {

        // Jo͂H
        if( CESystem::get()->getOption()->getExportCamera() ) {

            if( !getCamera( sceneItem, cfNode ) ) {

                Application app;
                app.LogMessage( L"err: getCamera." );
                return false;				

            }

        }

    }

    // e
    SIObject parentSIObject( sceneItem.GetParent() );
    if( parentSIObject.IsValid() ) {

        SceneItem parentSceneItem = ( parentSIObject );
        if( parentSceneItem.IsValid() ) {

            while( !CEUtil::isGetObject( parentSceneItem ) ) {

                // eV[[gɂȂ߂
                if( CEUtil::isRootObject( parentSceneItem ) ) {

                    break;

                }

                parentSceneItem = parentSceneItem.GetParent();

            }

            // eZNgXgɓĂꍇ̂ݐe
            if( CESystem::get()->isSelect( parentSceneItem ) ) {

                CEUtil::getRootPath( parentSceneItem, parentId );			

            }

        }

    }

    // e킩ȂƂScene_RootɂĂ
    if( *parentId == "" ) {

        *parentId = "Scene_Root";

    }

    // SRT擾
    bool local = true;
    if( *parentId == "Scene_Root" ) {

        local = false;

    }

    if( !getSRT( sceneItem, local, cfNode ) ) {	

        return false;    

    }		

    // JX^p[^擾
    Collada::Extra* cfExtra = new Collada::Extra();
    cfExtra->mProfile = "SICOLLADA_EXPORTER";
    if( !CEUtil::getCustomParameterBySceneItem( sceneItem, cfExtra ) ) {

        CE_SAFE_DELETE( cfExtra );
        return false;

    }
    cfNode->mExtra = cfExtra;

    return true;
}

// XSĨCg^Cv
enum XSI_LightType {

    XSI_LightType_Point = 0,
    XSI_LightType_Infinite = 1,
    XSI_LightType_Spot = 2,	

};

// Cg擾
bool CEObject::getLight( SceneItem& sceneItem,
                        Collada::Node* cfNode )
{
    Light light( sceneItem );
    VALID_FALSE( light );

    Parameter paramType = ProjectItem( light ).GetParameter( "Type" );
    const int tempType = paramType.GetValue( 0 );
    const XSI_LightType type = static_cast< XSI_LightType >( tempType );

    // color擾
    Collada::Color color( 1.0f, 1.0f, 1.0f, 1.0f );
    {
        CRef softLight = light.GetShaders().GetItem( L"soft_light" );
        VALID_FALSE( softLight );

        Parameter paramColor = ProjectItem( softLight ).GetParameter( "color" );
        VALID_FALSE( paramColor );

        CEUtil::getRGBValue( paramColor, &color );

        if( paramColor.IsAnimated() ) {

            Parameter paramR = paramColor.GetParameter( "red" );
            Parameter paramG = paramColor.GetParameter( "green" );
            Parameter paramB = paramColor.GetParameter( "blue" );

        }
    }

    string id;
    CEUtil::getRootPath( sceneItem, &id );

    const string name = sceneItem.GetName().GetAsciiString();

    Collada::Light* cfLight = new Collada::Light();
    cfLight->mId = id;
    cfLight->mName = name;
    cfLight->mColor = color;

    switch( type ) {

    case XSI_LightType_Point:
        cfLight->mType = Collada::Light::LightType_POINT;
        break;

    case XSI_LightType_Infinite:
        cfLight->mType = Collada::Light::LightType_DIRECTIONAL;
        break;

    case XSI_LightType_Spot:
        cfLight->mType = Collada::Light::LightType_SPOT;	
        break;	

    }

    CESystem::get()->getCollection()->mLights.push_back( cfLight );

    const string url = "#" + id;

    cfNode->mInstanceType = Collada::Node::InstanceType_LIGHT;	
    cfNode->mInstanceLight = new Collada::InstanceLight( url );

    return true;
}

// J擾
bool CEObject::getCamera( SceneItem& sceneItem,
                         Collada::Node* cfNode )
{
    Camera camera( sceneItem );
    VALID_FALSE( camera );

    Parameter paramFOV = ProjectItem( camera ).GetParameter( "fov" );
    Parameter paramFOVType = ProjectItem( camera ).GetParameter( "fovtype" );
    Parameter paramAspect = ProjectItem( camera ).GetParameter( "aspect" );	
    Parameter paramNear = ProjectItem( camera ).GetParameter( "near" );		
    Parameter paramFar = ProjectItem( camera ).GetParameter( "far" );			

    const float fov = paramFOV.GetValue( 0 );
    const float fovType = paramFOVType.GetValue( 0 ); // 
    const float aspect = paramAspect.GetValue( 0 );
    const float znear = paramNear.GetValue( 0 );
    const float zfar = paramFar.GetValue( 0 );	

    string id;
    CEUtil::getRootPath( sceneItem, &id );

    const string name = sceneItem.GetName().GetAsciiString();

    Collada::Camera* cfCamera = new Collada::Camera();
    cfCamera->mId = id;
    cfCamera->mName = name;
    cfCamera->mYfov = fov;
    cfCamera->mAspectRatio = aspect;
    cfCamera->mZNear = znear;
    cfCamera->mZFar = zfar;

    CESystem::get()->getCollection()->mCameras.push_back( cfCamera );

    const string url = "#" + id;

    cfNode->mInstanceType = Collada::Node::InstanceType_CAMERA;
    cfNode->mInstanceCamera = new Collada::InstanceCamera( url );

    return true;
}

// WIg擾
bool CEObject::getGeometry( SceneItem& sceneItem,
                           Collada::Node* cfNode )							 
{
    // WIg̖OShape̖OƓ
    const string geometryId( cfNode->mId );

    // WIg{̂擾
    if( !CESystem::get()->getCEGeometry()->get( sceneItem ) ) {

        Application app;
        app.LogMessage( L"err: can not get geometry infomation." );                    
        return false;

    }

    // ǉWIg
    Collada::Geometry* colladaGeometry = CESystem::get()->getCollection()->getGeometry( geometryId );
    if( !colladaGeometry ) {

        // ݂͑͂
        return false;

    }

    // polylistȌꍇ肤ibṼ|SȌꍇȂǁj
    if( colladaGeometry->mPolyLists.size() == 0 ) {

        // G[ł͂Ȃ
        return true;

    }

    // polylist̐InstanceMaterialǉ        
    vector< Collada::InstanceMaterial > instanceMaterials;

    const int polylistNumber = colladaGeometry->mPolyLists.size();
    for( int i=0; i<polylistNumber; i++ ) {

        Collada::PolyList* polylist = colladaGeometry->mPolyLists[ i ];

        instanceMaterials.push_back( Collada::InstanceMaterial( polylist->mMaterial, // symbol
            "#" + polylist->mMaterial ) ); // target

    }

    // InstanceGeometryNodeɒǉ
    cfNode->mInstanceType = Collada::Node::InstanceType_GEOMETRY;
    cfNode->mInstanceGeometry = new Collada::InstanceGeometry( "#" + geometryId, // sourceigeometry̖Oj
        instanceMaterials );

    // ۂɎgĂ}eA̔z
    CRefArray materialArray;
    {
        PolygonMesh polygonMesh( X3DObject( sceneItem ).GetActivePrimitive().GetGeometry() );
        CGeometryAccessor ga = polygonMesh.GetGeometryAccessor();
        materialArray = ga.GetMaterials();
    }

    // }eA擾
    // ̎ł͏ȗ

    return	true; 
}

// SRT擾
bool CEObject::getSRT( SceneItem&	sceneItem,
                      const bool local,
                      Collada::Node* cfNode )
{
    Collada::Float3 scale;
    Collada::Float4 rotateX;
    Collada::Float4 rotateY;
    Collada::Float4 rotateZ;
    Collada::Float3 translate;

    if( !CEUtil::getSRT( sceneItem, local, &scale, &rotateX, &rotateY, &rotateZ, &translate ) ) {

        Application app;
        app.LogMessage( L"err: getSRT()." );                                
        return false;

    }

    cfNode->mTranslate = translate;
    cfNode->mRotationX = rotateX;
    cfNode->mRotationY = rotateY;
    cfNode->mRotationZ = rotateZ;
    cfNode->mScale = scale;

    return	true;
}
