// Copyright(C)SEGA 
#include "CESystem.h" 
#include "CEUtil.h"
#include "CEObject.h"
#include "CEMacro.h"
#include "CEOption.h"
#include "CEGeometry.h"
#include "CEWindows.h"

#include "Collada/Collection.h"
#include "Collada/Macro.h"
#include "Collada/VisualScene.h"
#include "Collada/Asset.h"
#include "Collada/Contributor.h"

#include "FileIO/Path.h"
#include "FileIO/Directory.h"

#include <xsi_application.h>
#include <xsi_model.h>
#include <xsi_parameter.h>

#include <sstream>
#include <iomanip>

using namespace std;
using namespace XSI;

// VOg̃CX^X
CESystem* CESystem::mInstance = NULL;

// RXgN^ 
CESystem::CESystem() : mCollection(NULL)
{
    mOption = new CEOption();
    mCollection = new Collada::Collection();
    mObject = new CEObject();
    mGeometry = new CEGeometry();
}

// fXgN^
CESystem::~CESystem()
{
    mCopyTextureArray.clear();
    mRefArraySelection.Clear();

    CE_SAFE_DELETE( mOption );
    CE_SAFE_DELETE( mCollection );
    CE_SAFE_DELETE( mObject );	
    CE_SAFE_DELETE( mGeometry );    
}

// CX^X擾
void CESystem::createInstance()
{
    mInstance = new CESystem();
}

// CX^X擾
void CESystem::destroyInstance()
{
    CE_SAFE_DELETE( mInstance );
}

// CX^X擾
CESystem* CESystem::get()
{
    return mInstance;
}

// Collection폜
void CESystem::clearCollection()
{
    CE_SAFE_DELETE( mCollection );
}

// 
bool CESystem::initialize()
{
    mCopyTextureArray.clear();
    mRefArraySelection.Clear();

    return	true;
}

// ColladaԂ
Collada::Collection* CESystem::getCollection()
{
    return mCollection;
}

// ZNgXg擾
CRefArray CESystem::getSelection()
{
    return mRefArraySelection;
}

// ZNgXgɒǉ
void CESystem::addSelection( CRef& ref )
{
    mRefArraySelection.Add( ref );
}

// ZNgXgɓĂ邩ׂ
bool CESystem::isSelect( CRef& ref ) const
{
    const int num = mRefArraySelection.GetCount();
    for( int i=0; i<num; i++ ) {

        if( mRefArraySelection[ i ] == ref ) {

            return true;

        }

    }

    return false;
} 

// ZNgXgNA
void CESystem::clearSelection()
{
    mRefArraySelection	= CRefArray();
}

// p[^[̐e擾ii[Ă΁j
CRef CESystem::getParameterParent( Parameter& parameter )
{
    const int parameterNumber = mParameterSetArray.size();
    for( int i=0; i<parameterNumber; i++ ) {

        if( parameter == mParameterSetArray[ i ]->mParameter ) {

            return mParameterSetArray[ i ]->mParent;

        }

    }

    return CRef();
}

// IvVԂ
CEOption* CESystem::getOption()
{
    return mOption;
}

// AZbg擾
void CESystem::getAsset( const string& option )
{
    // ColladaAsset𐶐Ēǉ
    Collada::Asset* asset = new Collada::Asset();
    mCollection->mAsset = asset;

    // UpdateԂݒ
    {
        int year = 0;
        int mon = 0;
        int day = 0;
        int hour = 0;
        int min = 0;
        int sec = 0;

        CEWindows::getTime( &year, &mon, &day, &hour, &min, &sec );

        string strYear;
        {
            ostringstream ss;
            ss << setw( 4 ) << year;
            strYear = ss.str();
        }

        string strMon;
        {
            ostringstream ss;
            ss << setw( 2 ) << mon;            
            strMon = ss.str();
        }

        string strDay;
        {
            ostringstream ss;
            ss << setw( 2 ) << day;
            strDay = ss.str();
        }

        string strHour;
        {
            ostringstream ss;
            ss << setw( 2 ) << hour;            
            strHour = ss.str();
        }

        string strMin;
        {
            ostringstream ss;
            ss << setw( 2 ) << min;
            strMin = ss.str();
        }

        string strSec;
        {
            ostringstream ss;
            ss << setw( 2 ) << sec;
            strSec = ss.str();
        }

        const string strUpdate = strYear + "-" + strMon + "-" + strDay + "T" + strHour + ":" + strMin + ":" + strSec + "Z";        

        asset->mCreated = strUpdate;
        asset->mModified = strUpdate;

    }

    // ColladaContributor𐶐Ēǉ
    Collada::Contributor* contributor = new Collada::Contributor();
    asset->mContributor = contributor;

    // comments
    contributor->mComments = "ColladaExporter export options: " + option;

    // AuthoringTool
    contributor->mAuthoringTool = "Softimage / ColladaExporter / " + string( __DATE__  ) + string( " " ) + string( __TIME__ );

    // [U[
    {
        string userName;		
        CEWindows::getUserName( &userName );
        contributor->mAuthor = userName;
    }
}

// V[̏擾
bool CESystem::getScene()
{
    // 擾IuWFNg̃Xg
    if( !CEUtil::createSelectList() ) {

        return false;

    }

    Collada::Collection* collection = CESystem::get()->getCollection();
    assert( collection );

    // VisualSceneǉ
    Collada::VisualScene* cfVisualScene = new Collada::VisualScene();

    // VisualScene̖OScene_Root̖OɂĂ
    string sceneRootName;
    {
        Application app;
        Model sceneRoot = app.GetActiveSceneRoot();
        sceneRootName = sceneRoot.GetName().GetAsciiString();
    }

    cfVisualScene->mId = sceneRootName;
    cfVisualScene->mName = sceneRootName;
    collection->mVisualScenes.push_back( cfVisualScene );

    // Sceneݒ
    collection->mScene.instanceVisualScene.mURL = "#" + cfVisualScene->mId;        

    // Assetݒ
    CESystem::getAsset( "" ); // ۂɂ̓IvVn

    // m[h擾ifoȂƂm[h͏o͂j
    if( !mObject->get() ) { 

        // G[I
        return false;

    }

    // TODO:
    // Colladãf[^gpȂUVZbg폜
    // i͓܂łj

    // I
    return	true;
}

// WIg擾NXԂ
CEGeometry* CESystem::getCEGeometry()
{
    return mGeometry;
}

// eNX`wtH_ɃRs[
bool CESystem::copyTexture( const string& texturePath )
{
    // eNX`Rs[tH_݂Ȃꍇ̓G[
    if( !FileIO::Directory::exists( texturePath ) ) {

        string msg = "err: can not find texture copy folder: " + texturePath + "\n";

        Application app;
        app.LogMessage( msg.c_str() );

        return false;

    }

    Application	app;

    const int textureNumber = mCopyTextureArray.size();
    for( int i=0; i<textureNumber; i++ ) {

        // Rs[̃t@C
        const string& srcFileName = mCopyTextureArray[ i ];

        string fileName;
        FileIO::Path::getFileName( srcFileName, &fileName );

        // Rs[̃t@C
        const string dstFileName = texturePath + "\\" + fileName;

        // Rs[s
        // Rs[tH_݂ȂA
        // ㏑֎~t@C̏ꍇ̓Rs[ɎsB
        // bZ[W{bNXoĕ\قǂB
        if( !CEWindows::copyFile( srcFileName, dstFileName ) ) {

            string msg = "err: can not copy texture file[ " + fileName + " ]\n";
            app.LogMessage( msg.c_str() );
            return false;

        }
        else {

            const string msg = "copy texture [" + fileName + "]\n";
            app.LogMessage( msg.c_str() );

        }

    }

    return true;
}

// Rs[eNX`Xgɒǉ
void CESystem::addCopyTexture( const string& filename )
{
    // dȂ悤ɂ
    const int textureNumber = mCopyTextureArray.size();
    for( int i=0; i<textureNumber; i++ ) {

        if( filename == mCopyTextureArray[ i ] ) {

            return;

        }

    }

    // dȂǉ
    mCopyTextureArray.push_back( filename );
}
