// Copyright(C)SEGA 
#include <string>
#include <xsi_application.h>
#include <xsi_selection.h>
#include <xsi_comapihandler.h>
#include <xsi_x3dobject.h>
#include <xsi_fcurve.h>
#include <xsi_primitive.h>
#include <xsi_geometry.h>
#include <xsi_imageclip2.h>
#include <xsi_source.h>
#include <xsi_material.h>
#include <xsi_model.h>
#include <xsi_statickinematicstate.h>
#include <xsi_kinematics.h>
#include <xsi_layer.h>

#include "CEUtil.h"
#include "CEMacro.h"
#include "CESystem.h"
#include "CEOption.h"

#include "Collada/Struct.h"
#include "Collada/Extra.h"
#include "Collada/ExtraAttribute.h"
#include "Collada/Variant.h"

#include <Windows.h>

using namespace XSI;
using namespace std;

// ~LT[擾
void CEUtil::getMixer( SIObject& siObject,
                      Mixer& mixer )
{
    Model model( siObject );
    if( model.HasMixer() ) {

        mixer = model.GetMixer();
        return;

    }

    // SceneRoot܂ŗĂ܂I
    if( CEUtil::isSceneRoot( siObject ) ) {

        return;

    }

    // eɑ΂ČĂяo
    getMixer( X3DObject( siObject.GetParent() ), mixer );
}

// ̒u
void CEUtil::replaceChar( string* buffer,
                         const char c1,
                         const char c2 )
{
    const int num = buffer->size();
    for( int i=0; i<num; i++ ) {

        if( (*buffer)[ i ] == c1 ) {

            (*buffer)[ i ]	= c2;

        }

    }
}

// l01͈̔͂ɂ
void CEUtil::setRange0To1( float* value )
{
    if( (*value) > 1.0f ) {

        (*value) = 1.0f;

    }
    else if( (*value) < 0.0f ) {

        (*value) = 0.0f;

    }
}

// gXtH[[V擾
MATH::CTransformation CEUtil::getTransformation( SceneItem& sceneItem )
{
    CRef ref( sceneItem );

    // Ll}eBNX擾
    X3DObject x3DObject( ref );
    Kinematics kinematics	= x3DObject.GetKinematics();
    KinematicState kinematicState = kinematics.GetGlobal();

    // gXtH[擾
    MATH::CTransformation transformation = kinematicState.GetTransform();

    return transformation;
}

// IuWFNgCMatrix擾
bool CEUtil::getCMatrix( SceneItem& sceneItem,
                        const bool local,
                        MATH::CMatrix4& matrix )
{
    CRef ref( sceneItem );

    // Ll}eBNX擾
    X3DObject	x3DObject( ref );
    VALID_FALSE( ref );

    Kinematics kinematics = x3DObject.GetKinematics();
    VALID_FALSE( kinematics );

    KinematicState	kinematicState; 

    if( local ) {

        kinematicState = kinematics.GetLocal();

    }
    else {

        kinematicState = kinematics.GetGlobal();

    }

    // gXtH[擾
    VALID_FALSE( kinematicState );    
    MATH::CTransformation transformation = kinematicState.GetTransform();

    matrix = transformation.GetMatrix4();

    return true;
}

// IuWFNgInvMatrix擾
bool CEUtil::getInvMatrix( SceneItem& sceneItem,
                          const bool local,
                          Collada::Matrix* cfMatrix )
{
    MATH::CMatrix4 matrix;
    if( !getCMatrix( sceneItem, local, matrix ) ) {

        return false;

    }

    // tsvZ
    if( !matrix.Invert( matrix ) ) {

        return false;

    }

    CMatrixToColladaMatrix( matrix, cfMatrix );

    return true;
}

// IuWFNgMatrix擾
bool CEUtil::getMatrix( SceneItem& sceneItem,
                       const bool local,
                       Collada::Matrix* cfMatrix )
{
    CRef ref( sceneItem );

    // Ll}eBNX擾
    X3DObject	x3DObject( ref );
    VALID_FALSE( ref );

    Kinematics kinematics = x3DObject.GetKinematics();
    VALID_FALSE( kinematics );

    KinematicState	kinematicState;

    if( local ) {
        kinematicState = kinematics.GetLocal();
    }
    else {
        kinematicState = kinematics.GetGlobal();
    }

    VALID_FALSE( kinematicState );    
    MATH::CTransformation transformation = kinematicState.GetTransform();
    MATH::CMatrix4 matrix = transformation.GetMatrix4();

    CMatrixToColladaMatrix( matrix, cfMatrix );

    return true;
}

// XSIMatrixColladaMatrixɕϊ
void CEUtil::CMatrixToColladaMatrix( const MATH::CMatrix4& matrix,
                                    Collada::Matrix* cfMatrix )
{
    // MATH::CMatrix4Collada
    MATH::CMatrix4 m;
    m.Transpose( matrix );

    for( int i=0; i<4; i++ ) {

        for( int j=0; j<4; j++ ) {

            cfMatrix->m[ i*4+j ] = static_cast< float >( m.GetValue( i, j ) );

        }

    }
}

// IuWFNgSRT擾
bool CEUtil::getSRT( SceneItem& sceneItem,
                    const bool local,
                    Collada::Float3* scale,
                    Collada::Float4* rotationX,
                    Collada::Float4* rotationY,
                    Collada::Float4* rotationZ,
                    Collada::Float3* translation )					  
{
    CRef ref( sceneItem );

    // Ll}eBNX擾
    X3DObject	x3DObject( ref );
    VALID_FALSE( ref );

    Kinematics kinematics = x3DObject.GetKinematics();
    VALID_FALSE( kinematics );

    KinematicState	kinematicState;
    if( local ) {

        kinematicState = kinematics.GetLocal();

    }
    else {

        kinematicState = kinematics.GetGlobal();	

    }

    // gXtH[擾
    VALID_FALSE( kinematicState );
    MATH::CTransformation	transformation	= kinematicState.GetTransform();

    // scale
    MATH::CVector3	cscale = transformation.GetScaling();

    // rotation
    MATH::CVector3	crotation	= transformation.GetRotationXYZAngles();

    // translation
    MATH::CVector3	ctranslation 	= transformation.GetTranslation();

    const float rx = static_cast< float >( MATH::RadiansToDegrees( crotation.GetX() ) );
    const float ry = static_cast< float >( MATH::RadiansToDegrees( crotation.GetY() ) );
    const float rz = static_cast< float >( MATH::RadiansToDegrees( crotation.GetZ() ) );    

    MATH::CRotation::RotationOrder rotationOrder = transformation.GetRotationOrder();

    // ߂lZbg
    *scale = CEUtil::getVector3d( cscale );
    *translation = CEUtil::getVector3d( ctranslation );

    // rotation͉]lĊi[
    switch( rotationOrder ) {

    case MATH::CRotation::siXYZ:
        rotationX->set( 1.0f, 0.0f, 0.0f, rx );
        rotationY->set( 0.0f, 1.0f, 0.0f, ry );
        rotationZ->set( 0.0f, 0.0f, 1.0f, rz );
        break;

    case MATH::CRotation::siXZY:
        rotationX->set( 1.0f, 0.0f, 0.0f, rx );
        rotationY->set( 0.0f, 0.0f, 1.0f, ry );
        rotationZ->set( 0.0f, 1.0f, 0.0f, rz ); 
        break;

    case MATH::CRotation::siYXZ:
        rotationX->set( 0.0f, 1.0f, 0.0f, rx );
        rotationY->set( 1.0f, 0.0f, 0.0f, ry );
        rotationZ->set( 0.0f, 0.0f, 1.0f, rz );
        break;

    case MATH::CRotation::siYZX:
        rotationX->set( 0.0f, 0.0f, 1.0f, rx );
        rotationY->set( 1.0f, 0.0f, 0.0f, ry );
        rotationZ->set( 0.0f, 1.0f, 0.0f, rz );
        break;

    case MATH::CRotation::siZXY:
        rotationX->set( 0.0f, 1.0f, 0.0f, rx );
        rotationY->set( 0.0f, 0.0f, 1.0f, ry );
        rotationZ->set( 1.0f, 0.0f, 0.0f, rz );
        break;

    case MATH::CRotation::siZYX:
        rotationX->set( 0.0f, 0.0f, 1.0f, rx );
        rotationY->set( 0.0f, 1.0f, 0.0f, ry );
        rotationZ->set( 1.0f, 0.0f, 0.0f, rz );
        break;

    }

    return	true;
}

// IuWFNgSQT擾
bool CEUtil::getSQT( SceneItem& sceneItem,
                    const bool local,
                    Collada::Float3* scale,    
                    Collada::Float4* quaternion,
                    Collada::Float3* translation,
                    const double frame )
{
    CRef ref( sceneItem );

    // Ll}eBNX擾
    X3DObject	x3DObject( ref );
    VALID_FALSE( ref );

    Kinematics kinematics = x3DObject.GetKinematics();
    VALID_FALSE( kinematics );

    KinematicState	kinematicState;
    if( local ) {

        kinematicState = kinematics.GetLocal();

    }
    else {

        kinematicState = kinematics.GetGlobal();	

    }

    // gXtH[擾
    VALID_FALSE( kinematicState );
    MATH::CTransformation	transformation	= kinematicState.GetTransform( frame );

    // scale
    MATH::CVector3	cscale = transformation.GetScaling();

    // rotation
    MATH::CQuaternion cquaternion = transformation.GetRotationQuaternion();
    cquaternion.Normalize();

    // translation
    MATH::CVector3	ctranslation = transformation.GetTranslation();

    // ߂lZbg
    *scale = CEUtil::getVector3d( cscale );
    quaternion->set( static_cast< float >( cquaternion.GetX() ),
        static_cast< float >( cquaternion.GetY() ),
        static_cast< float >( cquaternion.GetZ() ),
        static_cast< float >( cquaternion.GetW() ) );
    *translation = CEUtil::getVector3d( ctranslation );

    return true;
}

// xNgFloat3^ŕԂ
Collada::Float3 CEUtil::getVector3d( MATH::CVector3&	vector3 )
{
    return Collada::Float3( static_cast<float>( vector3.GetX() ),
        static_cast<float>( vector3.GetY() ),
        static_cast<float>( vector3.GetZ() ) );
}

// Scene_RootǂԂ
bool CEUtil::isSceneRoot( SIObject &	SIObj )
{
    // e擾
    SIObject parent = SIObj.GetParent();
    VALID_TRUE( parent );

    // eType"Scene"Ȃ"Scene_Root"
    string parentType = parent.GetType().GetAsciiString();

    if( parentType == "Scene" ) {

        return	true;

    }
    return	false;
}

// RootɂIuWFNgǂԂ
bool CEUtil::isRootObject( SIObject&	object )
{
    // SceneRootȂtrue
    if( isSceneRoot( object ) ) {

        return	true;

    }

    // e擾
    SIObject parent = object.GetParent();

    // eSceneRoot?
    if( isSceneRoot( parent ) ) {

        return	true;

    }
    return	false;
}

// 擾IuWFNg̃Xg
bool CEUtil::createSelectList()
{
    // NA
    CESystem::get()->clearSelection();

    Application app;
    Selection selection = app.GetSelection();

    // IԂۑ(Selection͎gȂ̂ŒӁj
    CRefArray currentSelection;
    for( int s=0; s<selection.GetCount(); s++ ) {

        currentSelection.Add( selection[ s ] );

    }

    if( CESystem::get()->getOption()->getExportAll() ) {

        CComAPIHandler handler( app.GetRef() );

        CValueArray valueArray;
        handler.Call( L"SelectAll", CValue(), valueArray );

        selection = app.GetSelection();

    }

    const unsigned selectionCount	= selection.GetCount();
    for( unsigned i=0; i<selectionCount; i++ ) {

        CRef	ref( selection.GetItem( i ) );
        SceneItem	sceneItem( ref );

        const CString type	= sceneItem.GetType();

        // Ԃ`FbN
        bool	bVisibility;        
        if( !getRootVisibility( sceneItem, bVisibility ) ) {

            return	false;

        }

        if( bVisibility ) {

            CESystem::get()->addSelection( ref );

        }

    }

    // ZNgXg̎q̂ȂŃu`Î̂擾
    CRefArray branchArray;
    getBranchItem( branchArray );

    for( int i=0; i<branchArray.GetCount(); i++ ) {

        CESystem::get()->addSelection( branchArray[ i ] );	 	

    }

    // IԂɖ߂
    selection.Clear();
    for( int s=0; s<currentSelection.GetCount(); s++ ) {

        selection.Add( currentSelection[ s ] );

    }

    return	true;
} 

// ZNgXg̎q̂ȂŃu`Î̂擾
void CEUtil::getBranchItem( CRefArray& branchArray )
{
    Application app;
    Selection	selection	= app.GetSelection();

    const int count	= selection.GetCount();
    for( int i=0; i<count; i++ ) {

        ProjectItem projectItem( selection.GetItem( i ) );	
        if( !projectItem.IsValid() ) {

            continue;

        }

        // Ԃ`FbN
        bool visibility;        
        SceneItem sceneItem( projectItem );
        if( !getRootVisibility( sceneItem, visibility ) ) {

            continue;

        }
        if( !visibility ) {

            continue;

        }

        // u`tOu`̂Ƃ͎q𖳏ɒǉ
        const siBranchFlag flag = projectItem.GetBranchFlag();
        if( flag == siBranch ) {

            // ׂĂ̎q擾
            CRefArray childArray;
            getAllChildren( projectItem, childArray );

            for( int c=0; c<childArray.GetCount(); c++ ) {

                branchArray.Add( childArray[ c ] );

            }

        }

    }
}

// ׂĂ̎q擾
void CEUtil::getAllChildren( ProjectItem& parent,
                            CRefArray& childArray )
{
    // ȉ擾	
    CRefArray childArrayLocal = X3DObject( parent ).GetChildren();
    for( int i=0; i<childArrayLocal.GetCount(); i++ ) {

        ProjectItem child = childArrayLocal[ i ];

        // Ԃ`FbN  
        bool visibility; 
        SceneItem sceneItem( child );
        if( !getRootVisibility( sceneItem, visibility ) ) {

            continue;

        }
        if( !visibility ) {

            continue; 

        }

        childArray.Add( childArrayLocal[ i ] );

        getAllChildren( child, childArray );

    }
}

// J[p[^擾
bool CEUtil::getRGBValue( Parameter& param,
                         Collada::Color* color )
{
    Parameter paramR = param.GetParameter( "red" );
    Parameter paramG = param.GetParameter( "green" );
    Parameter paramB = param.GetParameter( "blue" );
    Parameter paramA = param.GetParameter( "alpha" );

    // ȂƂRGBȂƃG[
    if( !paramR.IsValid() || !paramG.IsValid() || !paramB.IsValid() ) {

        return	false;

    }

    color->r = paramR.GetValue();
    color->g = paramG.GetValue(); 
    color->b = paramB.GetValue();

    // Alpha͎Ƃ͌Ȃ
    if( paramA.IsValid() ) {

        color->a = paramA.GetValue();

    }
    else {

        color->a = 1.0f;

    }

    return	true;
}

// 擾IuWFNgH
bool CEUtil::isGetObject( SceneItem& sceneItem )
{
    VALID_FALSE( sceneItem );

    // ^Ń`FbN
    if( !checkType( sceneItem ) ) {

        return	false;

    }

    // H
    bool visibility	= true;
    getVisibility( sceneItem, visibility );
    if( !visibility ) {

        return	false;

    }

    return	true;
}

// SceneItem擾Ώۂ̃^Cvׂ
bool CEUtil::checkType( SceneItem& sceneItem )
{
    const CString type	= sceneItem.GetType();

    // JoH
    if( CESystem::get()->getOption()->getExportCamera() ) {

        if( type == L"camera" ||
            type == L"CameraRoot" ||
            type == L"CameraInterest" ||
            type == L"#model" ) {

                return true;

        }

    }

    // CgoH
    if( CESystem::get()->getOption()->getExportLight() ) {

        if( type == L"light" ||
            type == L"LightRoot" ||
            type == L"SpotRoot" ) {

                return true;

        }

    }

    if( type == L"polymsh"	||
        type == L"poly" 	||
        type == L"null" 	||
        type == L"bone"		||
        type == L"crvlist"	||
        type == L"arc"		||
        type == L"circle"	||
        type == L"cone"		||
        type == L"cube"		||
        type == L"cylinder"	||
        type == L"disc"		||
        type == L"grid"		||
        type == L"sphere"	||
        type == L"spiral"	||
        type == L"square"	||
        type == L"torus"	||
        type == L"root" 	||
        type == L"eff" 		||
        type == L"#model" )
    {

        return	true;

    }

    return false;
}

// [g܂ł̂ǂꂩȂH(܂)
bool CEUtil::getRootVisibility( SceneItem& checkSceneItem,
                               bool& rootVisibility )
{
    rootVisibility = true;
    SceneItem sceneItem = checkSceneItem;

    while( 1 ) {

        if( !sceneItem.IsValid() ) {

            return	true;

        }

        // SceneRoot܂ōsI
        if( isSceneRoot( sceneItem ) ) {

            return	true;

        }

        // H
        bool	bVisibility	= true;
        if( !getVisibility( sceneItem, bVisibility ) ) {

            return	false;

        }

        // sȂ̂I
        if( !bVisibility ) {

            rootVisibility	= false;
            return	true;

        }

        // e
        sceneItem	= sceneItem.GetParent();

        // eIXgɓĂȂI
        if( !CESystem::get()->isSelect( sceneItem ) ) {

            return true;

        }

    }

    return	true;
}

// ^sԂ擾
bool CEUtil::getVisibility( SceneItem&	sceneItem,
                           bool& visibility )
{
    // C[Ԃ`FbN
    bool visibleLayer	= true;
    if( !getLayer( sceneItem, visibleLayer ) ) {
        return	false;
    }

    // s?
    if( !visibleLayer ) {
        visibility	= false;
        return	true;
    }

    // HideԂ擾
    bool hide	= false;
    if( !getHide( sceneItem, &hide ) ) {
        return	false;
    }

    // HideȂ疳
    if( hide ) {
        visibility	= false;
        return	true;
    }

    visibility	= true;

    return	true;
}

// HideԂ擾
bool CEUtil::getHide( SceneItem& sceneItem,
                     bool* hide )
{
    *hide = false;

    CRefArray refArrayProperties( sceneItem.GetProperties() ); 
    if( !refArrayProperties.IsValid() ) {

        return false;

    }

    Property property;
    refArrayProperties.Find( L"visibility", property );

    // ǂNX^visibilityVisibility炵
    // visibilityŎsVisibilityŎ擾Ă݂
    if( !property.IsValid() ) {

        refArrayProperties.Find( L"Visibility", property );

    }

    if( !property.IsValid() ) {    

        if( isSceneRoot( sceneItem ) ) {

            *hide	= true;
            return	true;	

        }
        else {
            return	false;
        }

    }

    CParameterRefArray refArrayParameters( property.GetParameters() );

    Parameter parameter = refArrayParameters.GetItem( L"viewvis" );
    VALID_FALSE( parameter );

    // SceneRoot̂Ƃ͖Ɍ 
    if( isSceneRoot( sceneItem ) ) {

        hide	= false;
        return	true;

    }

    const bool viewVisibility = parameter.GetValue();

    if( viewVisibility ) {

        *hide = false;

    }
    else {

        *hide = true;

    }

    return	true;
}

// C[̏ԂԂ
bool CEUtil::getLayer( ProjectItem&	projectItem,
                      bool& visibleLayer )
{
    // I[i[擾
    CRefArray owners = projectItem.GetOwners();

    const int num = owners.GetCount();
    for( int i=0; i<num; i++ ) {

        // I[i[̓C[H    
        if( !owners[ i ].IsA( siLayerID ) ) {

            continue;

        }

        Layer layer( owners[ i ] );

        // C[̃p[^擾
        Parameter	paramViewVisibility = layer.GetParameter( "viewvis" );
        VALID_FALSE( paramViewVisibility );

        const int visibility = paramViewVisibility.GetValue();

        // C[͉Ԃɂ邩H 
        if( visibility == 0 ) {

            visibleLayer = false;

        }
        else {

            visibleLayer = true;

        }

    }

    return	true;
}

// Root܂ł̃pXԂ
void CEUtil::getRootPath( SIObject& object, string* path )
{
    // e擾
    SIObject parent = object.GetParent();

    // e擾ΏۂłȂ炳ɏ
    SceneItem sceneItemParent( parent );
    while( !isGetObject( sceneItemParent ) ) {

        sceneItemParent = sceneItemParent.GetParent();

        // ԏ܂ōsĂ܂I
        if( isSceneRoot( sceneItemParent ) ) {

            // ̂Ƃ͐eȂ
            *path = object.GetName().GetAsciiString();            
            return;

        }

    }    

    // Root̃IuWFNgȂ玩̖OԂ
    if( isRootObject( object ) ) {

        *path = object.GetName().GetAsciiString();        
        return;

    }

    // ̖O
    const string name = object.GetName().GetAsciiString();

    // eZNgXgɓĂȂ炻܂
    if( !CESystem::get()->isSelect( parent ) ) {

        *path = object.GetName().GetAsciiString();
        return;

    }

    // e̖O
    string parentName;
    getRootPath( parent, &parentName );

    // Collada̋؂"/"Ȃ̂Œ
    *path = parentName + "/" + name;
}

// Properties(CRefArray)JX^p[^擾
bool CEUtil::getCustomParameterByProperties( CRefArray& properties,
                                            Collada::Extra* cfExtra )
{
    Application app;

    const int count = properties.GetCount();
    for( int i=0; i<count; i++ ) {

        if( properties[ i ].GetClassID() != siCustomPropertyID ) {		

            continue;

        }

        Property property = properties[ i ];

        // vpeBJX^p[^擾
        if( !getCustomParameterByProperty( property, cfExtra ) ) {

            delete cfExtra;
            cfExtra = NULL;

            app.LogMessage( L"err: getCustomParameter()." );                            
            return false;

        }

    }

    return	true;
}

// PropertyJX^p[^擾
bool CEUtil::getCustomParameterByProperty( Property& property,
                                          Collada::Extra* cfExtra )
{
    // prefix͖
    return getCustomParameterByParameterArray( property.GetParameters(), "", cfExtra );
}

// ParameterArrayJX^p[^擾
bool CEUtil::getCustomParameterByParameterArray( CParameterRefArray& parameterArray,
                                                const string& prefix,
                                                Collada::Extra* cfExtra )
{
    const int parameterNumber = parameterArray.GetCount();

    vector< Collada::ExtraAttribute* > extraAttributeArray;

    for( int i=0; i<parameterNumber; i++ ) {

        Parameter	param	= parameterArray[ i ];
        if( !param.IsValid() ) {

            continue;

        }

        // GetName"Double Size"ȂǕ\p̖OԂĂĂ܂̂ŃXNvgp̖Og
        const string name = param.GetScriptName().GetAsciiString();

        // Cӂ̌^̃p[^擾
        Collada::ExtraAttribute* extraAttribute = new Collada::ExtraAttribute();
        if( getCustomParameterByParameter( parameterArray, param, param.GetScriptName(), extraAttribute ) ) {

            extraAttributeArray.push_back( extraAttribute );

        }
        else {

            delete extraAttribute;
            extraAttribute = NULL;

        }

    }

    const int extraAttributeNumber = extraAttributeArray.size();	

    // J[܂Ƃ߂
    vector< Collada::ExtraAttribute* > newExtraAttributeArray;	

    // J[̖O
    vector< string > colorNameArray;	

    int e=0;
    while( e < extraAttributeNumber ) {

        // cQȉɂȂǉďI
        if( e >= extraAttributeNumber - 2 ) {

            while( e < extraAttributeNumber ) {		

                newExtraAttributeArray.push_back( extraAttributeArray[ e ] );
                e++;

            }
            break;

        }

        string id( extraAttributeArray[ e ]->mId );

        bool findColor = false;

        if( id[ id.size()-1 ] == 'R' ) {

            string idG( id );
            idG[ idG.size()-1 ] = 'G';

            string idB( id );
            idB[ idB.size()-1 ] = 'B';			

            if( idG == extraAttributeArray[ e+1 ]->mId &&
                idB == extraAttributeArray[ e+2 ]->mId ) {

                    const float r = extraAttributeArray[ e ]->mValue->mFloatValue;
                    const float g = extraAttributeArray[ e+1 ]->mValue->mFloatValue;
                    const float b = extraAttributeArray[ e+2 ]->mValue->mFloatValue;

                    Collada::ExtraAttribute* extraAttribute = new Collada::ExtraAttribute();

                    // VO
                    id = id.substr( 0, id.size()-1 );
                    string colorName( id );
                    extraAttribute->mId = colorName;
                    extraAttribute->mName = colorName;

                    colorNameArray.push_back( colorName );

                    Collada::Variant* variant = new Collada::Variant();
                    variant->setColorValue( Collada::Color( r, g, b, 1.0f ) );
                    extraAttribute->mValue = variant;
                    newExtraAttributeArray.push_back( extraAttribute );

                    // J[vf͍폜
                    CE_SAFE_DELETE( extraAttributeArray[ e ] );
                    CE_SAFE_DELETE( extraAttributeArray[ e+1 ] );
                    CE_SAFE_DELETE( extraAttributeArray[ e+2 ] );				

                    findColor = true;
            }

        }

        // J[Ri߂
        if( findColor ) {

            e+=3;

        }
        // J[ȊOȂPǉ邾
        else {

            newExtraAttributeArray.push_back( extraAttributeArray[ e ] );	
            extraAttributeArray[ e ] = NULL;
            e++;

        }

    }
    extraAttributeArray.clear();

    // Extra֒ǉ
    const int newExtraAttributeNumber = newExtraAttributeArray.size();
    for( int e=0; e<newExtraAttributeNumber; e++ ) {

        // color̂ȂɖÔ݂āAľ^floatȂ疳
        bool findColor = false;
        const int colorNameNumber = colorNameArray.size();
        for( int c=0; c<colorNameNumber; c++ ) {

            if( colorNameArray[ c ] == newExtraAttributeArray[ e ]->mId &&
                newExtraAttributeArray[ e ]->mValue->mVariantType == Collada::Variant::VariantType_FLOAT ) {

                    findColor = true;
                    break;

            }

        }
        if( findColor ) {

            continue;

        }

        // prefixîƂj
        const string name = prefix + newExtraAttributeArray[ e ]->mId;
        newExtraAttributeArray[ e ]->mId = name;
        newExtraAttributeArray[ e ]->mName = name;

        if( !cfExtra->addExtraAttribute( newExtraAttributeArray[ e ] ) ) {

            CE_SAFE_DELETE( newExtraAttributeArray[ e ] );

        }
        else {

            newExtraAttributeArray[ e ] = NULL;

        }

    }

    return true;
}

// ParameterJX^p[^擾
bool CEUtil::getCustomParameterByParameter( CParameterRefArray& parameterArray,
                                           Parameter& param,
                                           CString& paramName,
                                           Collada::ExtraAttribute* extraAttribute )

{
    // GetName"Double Size"ȂǕ\p̖OԂĂĂ܂
    const string name = paramName.GetAsciiString();

    // O
    extraAttribute->mId = name;
    extraAttribute->mName = name;

    // l
    Collada::Variant* variant = new Collada::Variant();
    extraAttribute->mValue = variant;

    const CValue::DataType	type = param.GetValueType();
    switch( type ) {

    case CValue::siColor4f:
        {
            // eNX`qĂꍇȂǂ͎擾łȂ
            bool findColor = false;
            Parameter paramR = param.GetParameters().GetItem( L"red" );
            Parameter paramG = param.GetParameters().GetItem( L"green" );
            Parameter paramB = param.GetParameters().GetItem( L"blue" );

            if( paramR.IsValid() && paramG.IsValid() && paramB.IsValid() ) {

                const float r = paramR.GetValue();
                const float g = paramG.GetValue();
                const float b = paramB.GetValue();
                Collada::Color color( r, g, b, 1.0f );
                variant->setColorValue( color );

                findColor = true;

            }

            // J[擾łȂƂ͔ɂĂ
            if( !findColor ) {

                Collada::Color color( 1.0f, 1.0f, 1.0f, 1.0f );
                variant->setColorValue( color );			

            }
        }
        break;

    case siInt4:
        {
            const long value = param.GetValue();
            variant->setIntValue( value );
        }
        break;

    case siUInt4:
        {
            const unsigned long value = param.GetValue();
            variant->setIntValue( value );
        }
        break;

    case siFloat:
        {
            // J[^łfloatɂȂĂ̂iJX^p[^̃J[Ƃj
            // łfloatƂĎ擾Čł܂Ƃ߂ĂB
            const float value = param.GetValue();
            variant->setFloatValue( value );
        }
        break;

    case siDouble:
        {
            const double value = param.GetValue();
            variant->setFloatValue( static_cast< float >( value ) );
        }
        break;

    case siBool:
        {
            const bool value = param.GetValue();
            variant->setBoolValue( value );
        }
        break;

    case siString:
        {
            CString value = param.GetValue();
            variant->setStringValue( value.GetAsciiString() );
        }
        break;

    }

    return true;
}

// ImageClipJX^p[^擾
bool CEUtil::getCustomParameterByImageClip( ImageClip2&	imageClip,
                                           Collada::Extra* cfExtra )
{
    // ImageClip -> ImageClip2ɕϊ
    ImageClip2	imageClip2( imageClip );
    VALID_FALSE( imageClip2 );

    // ImageClip2̃\[X(ImageSource)擾
    Source	source( imageClip2.GetSource() );
    VALID_FALSE( source );

    // SourcẽvpeBɃJX^p[^tĂ
    CRefArray properties = source.GetProperties();
    VALID_FALSE( properties );

    if( !getCustomParameterByProperties( properties, cfExtra ) ) {

        Application app;
        app.LogMessage( L"err: getCustomParameterByProperties()." );
        return false;

    }

    return true;
}

// SceneItemJX^p[^擾
bool CEUtil::getCustomParameterBySceneItem( SceneItem& sceneItem,
                                           Collada::Extra* cfExtra )
{
    // vpeB擾
    CRefArray properties = sceneItem.GetProperties();
    VALID_FALSE( properties );

    if( !getCustomParameterByProperties( properties, cfExtra ) ) {

        Application app;
        app.LogMessage( "err: getCustomParameterByProperties()." );
        return false;

    }
    return	true;
} 

