// Copyright(C)SEGA 
#include <xsi_pluginregistrar.h>
#include <xsi_application.h>
#include <xsi_command.h>
#include <xsi_context.h>
#include <xsi_argument.h>
#include <xsi_selection.h>

#include <string>

#include <Windows.h>

#include "Collada/Collection.h"
#include "Collada/XML.h"
#include "Collada/Test.h"

#include "CEUtil.h"
#include "CESystem.h"
#include "CEOption.h"

#include "FileIO/Path.h"

using namespace std;
using namespace XSI;


// DllMain
BOOL APIENTRY DllMain( HANDLE hModule, 
                      DWORD  ul_reason_for_call,  
                      LPVOID lpReserved
                      )
{ 
    return TRUE;
} 

// DllCanUnloadNow
STDAPI DllCanUnloadNow(void) 
{ 
    return S_OK;       
}

// XSILoadPluginR[obN
XSIPLUGINCALLBACK CStatus XSILoadPlugin( PluginRegistrar& in_reg )
{
    in_reg.PutAuthor( L"Tatefuku Hiroshi" );

    in_reg.PutName( L"SIColladaExporter" );
    in_reg.PutVersion( 1, 0 );

    Application app;

    // Ƀ[hĂ폜B
    if( app.GetCommands().GetItem( L"SIColladaExporter" ).IsValid() ) {

        app.RemoveCommand( in_reg.GetName() );

    }

    // R}ho^
    in_reg.RegisterCommand( L"SIColladaExporter", L"SIColladaExporter" );

    // R}h[hɃbZ[W\
    app.LogMessage( CString("load plug-in:") + in_reg.GetName() );

    return CStatus::OK;
}

// XSIUnloadPluginR[obN
CStatus XSIUnloadPlugin( PluginRegistrar& in_reg )
{
    Application app;

    // R}hA[hɃbZ[W\
    app.LogMessage( CString("unload plug-in:") + in_reg.GetName() );

    return	true;
}

// 
XSIPLUGINCALLBACK CStatus SIColladaExporter_Init( CRef& in_ctxt )
{
    // ̃R}ḧݒ肷
    Context ctxt( in_ctxt );
    Command cmd;
    cmd = ctxt.GetSource();
    cmd.PutDescription( L"" ); // Softimagẽ[U[C^[tF[Xŕ\镶i󕶎j
    cmd.EnableReturnValue( true ); // ߂lgp

    ArgumentArray args;
    args = cmd.GetArguments();
    args.Add( L"OutputFileName" ); // o̓t@C
    args.Add( L"ExportAll" ); // So͂邩AÎo͂邩
    args.Add( L"ExportModel" ); // fo͂邩
    args.Add( L"ExportCamera" ); // Jo͂邩
    args.Add( L"ExportLight" ); // Cgo͂邩
    args.Add( L"ExportShape" ); // VFCvo͂邩 
    args.Add( L"ExportAnim" ); // Aj[Vo͂邩

    return CStatus::OK;
}

// s֐
XSIPLUGINCALLBACK CStatus SIColladaExporter_Execute( CRef& in_ctxt )
{
    Context ctxt( in_ctxt );
    Application app;

    // 擾
    CValueArray args = ctxt.GetAttribute( L"Arguments" );

    // ̌`FbN
    if( args.GetCount() != 7 ) {

        app.LogMessage( "err: arguments number is wrong." );
        return	CStatus::Fail;  

    }

    int index = 0;

    // FileName
    string filename = CString( args[index++] ).GetAsciiString();

    // ExportAll
    const bool exportAll = ((bool)args[index++]) ? true : false;

    // fGNX|[gH
    const bool exportModel = ((bool)args[index++]) ? true : false;

    // JGNX|[gH
    const bool exportCamera = ((bool)args[index++]) ? true : false;	

    // CgGNX|[gH
    const bool exportLight = ((bool)args[index++]) ? true : false;		

    // VFCvGNX|[gH
    const bool exportShape = ((bool)args[index++]) ? true : false;	

    // AjGNX|[gH
    const bool exportAnim = ((bool)args[index++]) ? true : false;

    // o̓t@CV[ƂĎg
    string sceneName;
    FileIO::Path::getFileNameWithoutExtension( filename, &sceneName );

    // o[W\
    {
        CString msg = L"[SIColladaExporter " + CString( __DATE__ ) + " " + CString( __TIME__ ) + L"]";
        app.LogMessage( msg );
    }

    // XSĨV[擾IuWFNg𐶐 
    CESystem::createInstance(); 

    // 
    if( !CESystem::get()->initialize() ) {

        CESystem::destroyInstance();
        ctxt.PutAttribute( L"ReturnValue", false );                
        return	CStatus::Fail;

    }

    // IvVݒ
    CEOption* option = CESystem::get()->getOption();
    option->setSceneName( sceneName );
    option->setExportAll( exportAll );    
    option->setExportModel( exportModel );            
    option->setExportCamera( exportCamera );
    option->setExportLight( exportLight );
    option->setExportShape( exportShape );            	
    option->setExportAnim( exportAnim );

    // V[擾
    if( !CESystem::get()->getScene() ) { 

        CESystem::destroyInstance(); 
        ctxt.PutAttribute( L"ReturnValue", false );                
        return	CStatus::Fail;

    }

    /*
    // o͂V[͕sSȂ̂ŃeXg͊OĂ܂
    //
    // V[o͑OɃeXg
    if( !Collada::Test::testCollection( CESystem::get()->getCollection() ) ) {

        Application app;
        app.LogMessage( "err: can not pass test." );

        // sf[^o
        CESystem::get()->getCollection()->save( "d:/err.dae" );

        CESystem::destroyInstance(); 
        ctxt.PutAttribute( L"ReturnValue", false );                 	
        return	CStatus::Fail;		

    }
    */

    if( !CESystem::get()->getCollection()->save( filename ) ) {

        Application app;
        app.LogMessage( L"err: can not write." );

        CESystem::destroyInstance();
        ctxt.PutAttribute( L"ReturnValue", false );                
        return	CStatus::Fail;

    }

    // Collectionj
    CESystem::get()->clearCollection();

    // o͐tH_
    string outputPath;
    FileIO::Path::getDirectoryName( filename, &outputPath );

    // XSĨV[擾IuWFNgj
    CESystem::destroyInstance();

    // GNX|[g
    app.LogMessage( L"export complete." );

    // ߂lZbg
    ctxt.PutAttribute( L"ReturnValue", true );

    return CStatus::OK;
}
